/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Michel Bruno and Guillaume Riviere
 *                 Adriana Danes : modify getAllDataSourceName() in order to
 * return a Hashtable which maps the jndi names to the datasoure names.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.container;

import java.util.Set;
import java.util.HashSet;
import java.util.Hashtable;
import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.mail.MailServiceImpl;
import org.objectweb.jonas.dbm.DataBaseServiceImpl;
import org.objectweb.jonas_ejb.container.JFactory;
import org.objectweb.jonas_ejb.deployment.api.BeanDesc;
import org.objectweb.jonas_ejb.deployment.api.ResourceRefDesc;
import org.objectweb.jonas_ejb.deployment.api.ResourceEnvRefDesc;

/**
 * MBean Class for general EJB Management 
 * MBean type: Standard
 * MBean model: Delegate (ManagableEjb: JFactory)
 */
public class JmxBeanFactory implements JmxBeanFactoryMBean {
   
    /**
     * The Bean to manage
     */
    protected JFactory ejbToManage = null;
    protected BeanDesc desc = null;
    protected String fileName = null;

    /**
     * Constructor
     * @param ManagableEjb the bean to manage
     */
    public JmxBeanFactory(JFactory ejbToManage) {
	this.ejbToManage = ejbToManage;
	this.fileName = ejbToManage.getContainer().getFileName();
	this.desc = ejbToManage.getDeploymentDescriptor();
    } 

    /**
     * @return String The Name of this JFactory
     */ 
    public String getEJBName() {
	return desc.getEjbName();
    }

    /**
     * @return Ejb File Name
     */
    public String getEJBFileName() {
	return fileName;
    }

    /**
     * @return Set The Name set of the DataSources
     */
    public Hashtable getAllDataSourceName() {
	Hashtable result = new Hashtable();
	
	DataBaseServiceImpl bdmServ = null;
	String jndiName = null;
	String datasourceName = null;

	ResourceRefDesc[] rrDesc = desc.getResourceRefDesc();
	for (int i = 0; i < rrDesc.length; i ++) {
	    if (rrDesc[i].isJdbc()) {
		// The bean uses a datasource
		// try to get the DataBase service
		if (bdmServ == null)
		    try {
			bdmServ = (DataBaseServiceImpl)ServiceManager.getInstance().getDataBaseService();
		    } catch(Exception e) {
			throw new ServiceException("DataBase Service not available");
		    }
		// get the DD provided JNDI name
		jndiName = rrDesc[i].getJndiName();
	 
		// determine the datasource name (normally may no be null)
		datasourceName = bdmServ.getDatasourceName(jndiName);
		if (datasourceName != null)
		    result.put(jndiName, datasourceName);
		else
		    result.put(jndiName, "");
	    }
	}
	return result;
    }

 
    /**
     * @return Set The Name set of the Connection Factories
     */   
    public Set getAllJMSConnectionFactoryName(){
	Set result= new HashSet();
	ResourceRefDesc[] rrDesc = desc.getResourceRefDesc();
	for (int i = 0; i < rrDesc.length; i ++) {
	    if ((rrDesc[i].getType() == javax.jms.TopicConnectionFactory.class)
		||(rrDesc[i].getType() == javax.jms.QueueConnectionFactory.class)) {
		result.add(rrDesc[i].getJndiName()); 
	    }
	}
	return result;
    }

    /**
     * @return Hashtable which maps the JNDI names provided by the DD to the Session Mail factyory resources
     * known by the Mail Service
     */
    public Hashtable getAllMailFactorySName(){
	Hashtable result = new Hashtable();

	MailServiceImpl mailServ = null;
	try {
	    mailServ = (MailServiceImpl)ServiceManager.getInstance().getMailService();
	} catch(Exception e) {
	    // Mail Service not available, mailServ null
	}
	String jndiName = null; // name provided by the DD
	String factoryName = null; // resource name 

	ResourceRefDesc[] rrDesc = desc.getResourceRefDesc();
	for (int i = 0; i < rrDesc.length; i ++) {
	    if (rrDesc[i].getTypeName().equals("javax.mail.Session")) {
		// The Bean uses a javax.mail.Session resource

		// get the DD provided JNDI name
		jndiName = rrDesc[i].getJndiName();


		// try to get the mail service
		if (mailServ != null) {				
		    // get the resource factory name (maybe null if no resource factory registered under the jndiName)
		    factoryName = mailServ.getFactoryName(jndiName);
		    if (factoryName != null)
			result.put(jndiName, factoryName);
		    else
			result.put(jndiName, "");
		} else {
		    // Mail Service not available
		    result.put(jndiName, "");
		}
	    }
	}
	return result;
    }
    /**
     * @return Hashtable which maps the JNDI names provided by the DD to the MimePartDataSource Mail factyory resources
     * known by the Mail Service
     */
    public Hashtable getAllMailFactoryMName() {
	Hashtable result = new Hashtable();

	MailServiceImpl mailServ = null; 
	try {
	    mailServ = (MailServiceImpl)ServiceManager.getInstance().getMailService();
	} catch(Exception e) {
	    // Mail Service not available, mailServ null
	}
	String jndiName = null;
	String factoryName = null;

	ResourceRefDesc[] rrDesc = desc.getResourceRefDesc();
	for (int i = 0; i < rrDesc.length; i ++) {
	    if (rrDesc[i].getTypeName().equals("javax.mail.internet.MimePartDataSource")) {
		// the Bean uses a javax.mail.internet.MimePartDataSource resource

		// get the DD provided JNDI name
		jndiName = rrDesc[i].getJndiName();

		if (mailServ != null) {
		    // get the resource factory name (maybe bull if no resource factory registered under the jndiName)
		    factoryName = mailServ.getFactoryName(jndiName);
		    if (factoryName != null) 
			result.put(jndiName, factoryName);
		    else
			result.put(jndiName, "");
		} else {
		    // Mail Service not available
		    result.put(jndiName, "");
		}
	    }
	}
	return result;
    }

    /**
     * @return Set The Name set of the JMS Destinations
     */  
    public Set getAllJMSDestinationName(){
	Set result= new HashSet();
	ResourceEnvRefDesc[] rrDesc = desc.getResourceEnvRefDesc();
	for (int i = 0; i < rrDesc.length; i ++) {
	    if ((rrDesc[i].getType() == javax.jms.Topic.class )
		||(rrDesc[i].getType() == javax.jms.Queue.class )) { 
		result.add(rrDesc[i].getJndiName()); 
	    }
	}
	return result;
    }

    /**
     * @return Integer The current instance pool size
     */  
    public  Integer getCurrentInstancePoolSize(){
	return new Integer(ejbToManage.getPoolSize());
    }

    /**
     * @return String the  JFactory Class
     */
    public String getEJBClass(){
	return desc.getEjbClass().getName();
    }

    /**
     * @return String the displayName of the bean, or bean name if not defined.
     */
    public String getDisplayName() {
	return desc.getDisplayName() != null ? desc.getDisplayName() : desc.getEjbName();
    }

    /**
     * @return String the JNDI Name of the bean.
     */
    public String getJndiName() {
	return desc.getJndiName();
    }

    /**
     * @return String the HomeClass of the bean.
     */
    public String getHomeClass() {
	if (desc.getHomeClass() != null) {
	    return desc.getHomeClass().getName();
	} else {
	    return null;
	}
    }

    /**
     * @return String the RemoteClass of the bean.
     */
    public String getRemoteClass() {
	if (desc.getRemoteClass() != null) {
	    return desc.getRemoteClass().getName();
	} else {
	    return null;
	}
    }

    /**
     * @return String the LocalHomeClass of the bean.
     */
    public String getLocalHomeClass() {
	if (desc.getLocalHomeClass() != null) {
	    return desc.getLocalHomeClass().getName();
	} else {
	    return null;
	}
    }

    /**
     * @return String the LocalClass of the bean.
     */
    public String getLocalClass() {
	if (desc.getLocalClass() != null) {
	    return desc.getLocalClass().getName();
	} else {
	    return null;
	}
    }

}
