/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jmx.sunri;

// JMX includes
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

// JOnAS includes
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.JonasAlreadyStartedException;
import org.objectweb.jonas.jmx.AbsJmxServiceImpl;

// Monolog
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;


/**
 * JMX Service implementation for SUU RI
 * Provides specific doStart() method
 * @author Adriana Danes.

 */

public class JmxServiceImpl extends AbsJmxServiceImpl {

    /**
     * Start the Service. 
     * Only need to create a RMI
     */
    public void doStart() throws ServiceException {       

	try {
	    // Creat a RMI Connector for the JMX agent
	    RMIConnectorImpl rmiConnector = new RMIConnectorImpl(jmxServer);
	    // Register the connector in JNDI
	    InitialContext ictx = new InitialContext();
	    ictx.bind("RMIConnector_"+jonasName, rmiConnector);
	    ictx.close();	
	}catch (javax.naming.NameAlreadyBoundException e) {
	    logger.log(BasicLevel.DEBUG, "Cannot start JMX service "+e);
	    throw new JonasAlreadyStartedException();
	} catch (Exception e) {
	    logger.log(BasicLevel.DEBUG, "Cannot start JMX service "+e);
	    throw new ServiceException("Cannot start JMX service", e);
	}
    }

    /**
     * Stop this service
     */
    public void doStop() {
	try {
	    InitialContext ictx = new InitialContext();
	    ictx.unbind("RMIConnector_"+jonasName);
	    ictx.close();
	} catch (Exception e) {
	    logger.log(BasicLevel.ERROR, "Cannot Unbind Jmx RMI Connector" + e);
	}

	// Remove internal references to the MBeanServer. 
	releaseJmxServer();

	logger.log(BasicLevel.DEBUG, "JMX Service stopped");
    }
}
