/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.mail.factory;

//import java
import java.rmi.RemoteException;
import java.util.Properties;

/**
 * This interface defines all the MBean methods provided by the JMail objects.
 * @author Florent Benoit
 * @author Ludovic Bert
 * Contributor(s):
 * Adriana Danes : Refactor code: rename the management methods (use straightforward
 * names, as they are to be used in a Management Console)
 */
public interface JMailMBean {

    /**
     * Get the name of the factory.
     * @return name of the mail factory.
     */
    public String getFactoryName();

    /**
     * Get the jndi name of the factory.
     * @return jndi name of the mail factory.
     */
    public String getName();

    /**
     * Set the jndi name of the factory.
     * @param jndi name of the mail factory.
     */
    public void setName(String name);

    /**
     * Get the type of the factory.
     * @return type of the mail factory.
     */
    public String getFactoryType();


    /**
     * Get the authentication properties.
     * @return properties of the authentication.
     */
    public Properties getAuthenticationProperties();

    /**
     * Set the authentication properties.
     * @param props the authentication properties.
     */
    public void setAuthenticationProperties(Properties props);

    /**
     * Get the session properties.
     * @return the properties of Session.
     */
    public Properties getSessionProperties();

    /**
     * Set the session properties.
     * @param props the Session properties.
     */
    public void setSessionProperties(Properties props);
}

