/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
  

package org.objectweb.jonas.resource;

import java.util.Vector;
import javax.resource.ResourceException;
import javax.resource.spi.ManagedConnection;
import javax.transaction.Synchronization;
import javax.transaction.xa.XAResource;

/**
 *  A ManagedConnection and its Information
 *
 * @author     sebastien.chassande@inrialpes.fr
 * @author     Eric.Hardesty@bull.com
 */
class MCInfo {
        /**
         *  The managedConnection
         */
        public ManagedConnection mc;

        /**
         *  The list of used Connections
         */
        public Vector usedCs = null;

        /**
         *  The event used for the later enlisting into transaction 
         */
        public RMEImpl rme = null;

        /**
         *  Is the the managedConnection is inside a local transaction
         */
        public boolean localTransaction = false;

        /**
         *  If local transaction is used, then here is the LocalXAWrapper to use
         *  instead of an XA object
         */
        public LocalXAWrapper lw = null;

        /**
         *  The Context linked to the managedConnection instance
         * There are three state possible
         * global transaction : ctx= the reference to the transaction instance
         * local transaction: ctx=null / localTransaction = true
         * other ctx = null;
         */
        public Object ctx;

        /**
         *  The current Synchronisation object  used for the later enlisting into 
         * the global transaction 
         */
        public Synchronization synchro = null;


        /**
         *  Constructor for the MCInfo object
         */
        public MCInfo(ManagedConnection _mc) {
                mc = _mc;
                localTransaction = false;
                usedCs = new Vector();
        }


        /**
         *  Gets the State attribute of the MCInfo object
         *
         *@return         The State value
         */
        public String getState(String prefix) {
                String res = prefix + "* mc=" + mc + "\n";
                res += prefix + "Context=" + ctx + "\n";
                res += prefix + "size of usedCs:" + usedCs.size() + "\n";
                for (int i = 0; i < usedCs.size(); i++) {
                        res += prefix + "\t" + usedCs.elementAt(i).toString() + "\n";
                }
                return res;
        }


        /**
         *  Gets the State attribute of the MCInfo object
         */
        public String getState() {
                return getState("");
        }

        
        /**
         *  Fowards the detroy call on the ManagedConnection
         */
        public void destroy() throws Exception {
                mc.destroy();
        }

        /**
         *  Fowards the getResource call to the associated ConnectionManager
         */
        public XAResource getXAResource() throws ResourceException {
                if (lw != null) {
                        return lw;
                } else {
                        return mc.getXAResource();
                }
        }
}


