/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 */
package org.objectweb.jonas.server;

// general Java imports
import java.util.Properties;

// JMX imports
import javax.management.Notification;
import javax.management.NotificationListener;
import javax.management.NotificationBroadcaster;
import javax.management.MBeanServerDelegate;
import javax.management.MBeanServer;
import javax.management.MBeanServerNotification;
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;
import javax.management.InstanceNotFoundException;

import org.objectweb.jonas.common.Log;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
/**
 * MBean interface allowing to reconfigure a JOnAS server
 * @author Adriana Danes
 * Contributor(s):
 */
public class JonasConfig implements JonasConfigMBean, NotificationListener {

    static private Logger logger = null;
    Properties serverProperties;
    MBeanServer jmxServer = null;

    // The value of the 'type' property in JonasObjectName which is used for all the MBeans 
    // associated to the JOnAS services
    static final private String SERVICE_TYPE = "service"; 
    
    public JonasConfig(Properties theProperties, MBeanServer theMBeanServer) {
        // get a logger for server traces
        logger = Log.getLogger(Log.JONAS_SERVER_PREFIX);

        serverProperties = new Properties(theProperties);
        jmxServer = theMBeanServer;

        // Add myself as listner to the MBeanServerDelegate MBean in order to receive
        // MBeanServerNotifications (null NotificationFilter, null handback object)
        try {
            ObjectName delegate = new ObjectName("JMImplementation:type=MBeanServerDelegate");
            jmxServer.addNotificationListener(delegate, this, null, null);
        } catch (MalformedObjectNameException mne) {
            // This should not occur
            mne.printStackTrace();
            throw new RuntimeException("Could not create MBeanServerDelegate ObjectName with String JMImplementation:type=MBeanServerDelegate. ");
        } catch (InstanceNotFoundException e) {
            // This should not occur
            e.printStackTrace();
            throw new RuntimeException("MBeanServerDelegate - JMImplementation:type=MBeanServerDelegate - does not match any of the registered MBeans.");
        }
        logger.log(BasicLevel.DEBUG, "JonasConfig MBean registered itself as listner to MBeanServerNotifications");
    }

    public void handleNotification(Notification notification, java.lang.Object handback) {
        if (notification instanceof MBeanServerNotification) {
            String notificationType = notification.getType();
            // the notification sender
            ObjectName notificationSender = ((MBeanServerNotification)notification).getMBeanName();
            if (notificationType.equals(MBeanServerNotification.REGISTRATION_NOTIFICATION)) {
                logger.log(BasicLevel.DEBUG, "Received Registration Notification from " + notificationSender.toString());
            }
            if (notificationType.equals(MBeanServerNotification.UNREGISTRATION_NOTIFICATION)) {
                logger.log(BasicLevel.DEBUG, "Received Unregistration Notification from " + notificationSender.toString());
            }
        }
    }
}
