/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.web.catalina40;


//import objectweb
import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

//import catalina
import org.apache.catalina.util.xml.XmlMapper;

import org.apache.catalina.util.xml.SaxContext;
import org.apache.catalina.util.xml.XmlAction;
import java.util.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 *  This class provides an implementation of a logged XmlMapper with our 
 * specified logger.
 * @author Ludovic Bert
 * @author Florent Benoit
 */
public class JXmlMapper extends XmlMapper {

    /**
     * The specific logger to use. It is the JOnAS logger that will replace 
     * the Tomcat logger.
     */
    private Logger logger = null;

    /**
     * Construct a new JXmlMapper.
     */
    public JXmlMapper() {
        logger = Log.getLogger(Log.JONAS_WEB_PREFIX);
    }

    /**
     * This method write to the JOnAS logger the specified message.
     * @param msg the text to write on the logger.
     */
    public void log(String msg) {
        logger.log(BasicLevel.DEBUG, msg);
    }

    /** Create an object using for a matching tag with the given class name
     */
    public XmlAction objectCreate( String classN ) {
        return new ObjectCreate( classN);
    }

    /** Create an object using an attribute value as the class name
        If no attribute use classN as a default.
     */
    public XmlAction objectCreate( String classN, String attrib ) {
        return new ObjectCreate( classN, attrib);
    }


}

/**
 * Create an object of the specified or override Java class name.
 */
class ObjectCreate extends XmlAction {
    String className;
    String attrib;

    /**
     * Create an object of the specified class name.
     *
     * @param classN Fully qualified name of the Java class to instantiate
     */
    public ObjectCreate(String classN) {

        className=classN;
    }

    /**
     * Create an object of the specified default class name, unless an
     * attribute with the specified name is present, in which case the value
     * of this attribute overrides the default class name.
     *
     * @param classN Fully qualified name of the Java class to instantiate
     *  if the specified attribute name is not present
     * @param attrib Name of the attribute that may contain a fully qualified
     *  name of a Java class that overrides the default
     */
    public ObjectCreate(String classN, String attrib) {

        className=classN;
        this.attrib=attrib;
    }

    public void start( SaxContext ctx) throws Exception {

        Stack st=ctx.getObjectStack();
        int top=ctx.getTagCount()-1;
        String tag=ctx.getTag(top);
        String classN=className;

        if( attrib!=null) {
            AttributeList attributes = ctx.getAttributeList( top );
            if (attributes.getValue(attrib) != null)
                classN= attributes.getValue(attrib);
        }

	Class c=Class.forName( classN );
	Object o=c.newInstance();
	st.push(o);
	if( ctx.getDebug() > 0 ) ctx.log("new "  + attrib + " " + classN + " "  + tag  + " " + o);
        
    }

    public void cleanup( SaxContext ctx) {
        Stack st=ctx.getObjectStack();
        String tag=ctx.getTag(ctx.getTagCount()-1);
        Object o=st.pop();
        if( ctx.getDebug() > 0 ) ctx.log("pop " + tag + " " + o.getClass().getName() + ": " + o);
    }

}
