/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Craig R. McClanahan for Tomcat 4.1.x
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.web.catalina41;

//import java
import java.lang.reflect.Constructor;

//import apache commons
import org.apache.commons.digester.Digester;
import org.apache.commons.digester.Rule;

//import catalina
import org.apache.catalina.Container;
import org.apache.catalina.Loader;
import org.apache.catalina.startup.ContextRuleSet;
import org.apache.catalina.startup.CopyParentClassLoaderRule;
import org.apache.catalina.startup.LifecycleListenerRule;
import org.xml.sax.Attributes;

/**
 * This class define a JOnAS rule.
 * This rule is used to customise a context if the context is a JOnASContext.
 * It is based upon the ContextRuleSet of catalina.
 * @author Craig R. McClanahan
 * @author Florent Benoit : Tomcat 4.1.x
 */

public class JContextRuleSet extends ContextRuleSet {

    /**
     * The Catalina WebContainer service
     */
    private CatalinaJWebContainerServiceImpl catalinaService = null;


    /**
     * Construct an instance of this <code>RuleSet</code> with the specified
     * matching pattern prefix.
     *
     * @param prefix Prefix for matching pattern rules (including the
     *  trailing slash character)
     * @param catalinaService the Catalina WebContainer service
     */
    public JContextRuleSet(String prefix, CatalinaJWebContainerServiceImpl catalinaService) {
        super(prefix);
        this.catalinaService = catalinaService;
    }


    // --------------------------------------------------------- Public Methods


    /**
     * <p>Add the set of Rule instances defined in this RuleSet to the
     * specified <code>Digester</code> instance, associating them with
     * our namespace URI (if any).  This method should only be called
     * by a Digester instance.</p>
     *
     * @param digester Digester instance to which the new Rule instances
     *  should be added.
     */
    public void addRuleInstances(Digester digester) {

        if (!isDefaultContext()) {
            digester.addObjectCreate(prefix + "Context",
                                     "org.apache.catalina.core.StandardContext",
                                     "className");
        } else {
            digester.addObjectCreate(prefix + "Context",
                                     "org.apache.catalina.core.StandardDefaultContext",
                                     "className");
        }
        digester.addSetProperties(prefix + "Context");
        if (!isDefaultContext()) {
            digester.addRule(prefix + "Context",
                             new CopyParentClassLoaderRule(digester));
            digester.addRule(prefix + "Context",
                             new SetCatalinaServiceRule(digester, catalinaService));
            digester.addRule(prefix + "Context",
                             new LifecycleListenerRule
                             (digester,
                              "org.apache.catalina.startup.ContextConfig",
                              "configClass"));
            digester.addSetNext(prefix + "Context",
                                "addChild",
                                "org.apache.catalina.Container");
        } else {
            digester.addSetNext(prefix + "Context",
                                "addDefaultContext",
                                "org.apache.catalina.DefaultContext");
        }
        digester.addCallMethod(prefix + "Context/InstanceListener",
                               "addInstanceListener", 0);

        digester.addObjectCreate(prefix + "Context/Listener",
                                 null, // MUST be specified in the element
                                 "className");
        digester.addSetProperties(prefix + "Context/Listener");
        digester.addSetNext(prefix + "Context/Listener",
                            "addLifecycleListener",
                            "org.apache.catalina.LifecycleListener");

        digester.addRule(prefix + "Context/Loader",
                         new CreateLoaderRule
                         (digester,
                          "org.apache.catalina.loader.WebappLoader",
                          "className"));
        digester.addSetProperties(prefix + "Context/Loader");
        digester.addSetNext(prefix + "Context/Loader",
                            "setLoader",
                            "org.apache.catalina.Loader");

        digester.addObjectCreate(prefix + "Context/Logger",
                                 null, // MUST be specified in the element
                                 "className");
        digester.addSetProperties(prefix + "Context/Logger");
        digester.addSetNext(prefix + "Context/Logger",
                            "setLogger",
                            "org.apache.catalina.Logger");

        digester.addObjectCreate(prefix + "Context/Manager",
                                 "org.apache.catalina.session.StandardManager",
                                 "className");
        digester.addSetProperties(prefix + "Context/Manager");
        digester.addSetNext(prefix + "Context/Manager",
                            "setManager",
                            "org.apache.catalina.Manager");

        digester.addObjectCreate(prefix + "Context/Manager/Store",
                                 null, // MUST be specified in the element
                                 "className");
        digester.addSetProperties(prefix + "Context/Manager/Store");
        digester.addSetNext(prefix + "Context/Manager/Store",
                            "setStore",
                            "org.apache.catalina.Store");

        digester.addObjectCreate(prefix + "Context/Parameter",
                                 "org.apache.catalina.deploy.ApplicationParameter");
        digester.addSetProperties(prefix + "Context/Parameter");
        digester.addSetNext(prefix + "Context/Parameter",
                            "addApplicationParameter",
                            "org.apache.catalina.deploy.ApplicationParameter");

        digester.addObjectCreate(prefix + "Context/Realm",
                                 null, // MUST be specified in the element
                                 "className");
        digester.addSetProperties(prefix + "Context/Realm");
        digester.addSetNext(prefix + "Context/Realm",
                            "setRealm",
                            "org.apache.catalina.Realm");

        digester.addObjectCreate(prefix + "Context/ResourceLink",
                                 "org.apache.catalina.deploy.ContextResourceLink");
        digester.addSetProperties(prefix + "Context/ResourceLink");
        digester.addSetNext(prefix + "Context/ResourceLink",
                            "addResourceLink",
                            "org.apache.catalina.deploy.ContextResourceLink");

        digester.addObjectCreate(prefix + "Context/Resources",
                                 "org.apache.naming.resources.FileDirContext",
                                 "className");
        digester.addSetProperties(prefix + "Context/Resources");
        digester.addSetNext(prefix + "Context/Resources",
                            "setResources",
                            "javax.naming.directory.DirContext");

        digester.addObjectCreate(prefix + "Context/Valve",
                                 null, // MUST be specified in the element
                                 "className");
        digester.addSetProperties(prefix + "Context/Valve");
        digester.addSetNext(prefix + "Context/Valve",
                            "addValve",
                            "org.apache.catalina.Valve");

        digester.addCallMethod(prefix + "Context/WrapperLifecycle",
                               "addWrapperLifecycle", 0);

        digester.addCallMethod(prefix + "Context/WrapperListener",
                               "addWrapperListener", 0);

    }


    /**
     * Are we processing a DefaultContext element?
     * @return true if it's a default context
     */
    protected boolean isDefaultContext() {
        return (prefix.endsWith("/Default"));
    }


}


/**
 * Rule that set to a context an embedded instance if the context is a
 * JOnAS context
 * @author Philippe Coq
 * @author Florent Benoit : JOnAS/Tomcat 4.1.x
 */

final class SetCatalinaServiceRule extends Rule {

    /**
     * The Catalina WebContainer service
     */
    private CatalinaJWebContainerServiceImpl catalinaService = null;

    /**
     * Constructor of the SetEmbeddedRule.
     * @param digester the Digester to use.
     * @param catalinaService The Catalina WebContainer service
     */
    public SetCatalinaServiceRule(Digester digester, CatalinaJWebContainerServiceImpl catalinaService) {
        super(digester);
        this.catalinaService = catalinaService;
    }

    /**
     * This method is called when the beginning of a matching XML element
     * is encountered.
     *
     * @param attributes The attribute list of this element
     * @throws Exception if it fails.
     */
    public void begin(Attributes attributes) throws Exception {

        // Look up the required parent class loader
        Object top = digester.peek();

        if (top instanceof JOnASStandardContext) {
            ((JOnASStandardContext) top).setCatalinaJWebContainerService(catalinaService);
        }
    }
}


/**
 * Rule that creates a new <code>Loader</code> instance, with the parent
 * class loader associated with the top object on the stack (which must be
 * a <code>Container</code>), and pushes it on to the stack.
 * @author Tomcat Catalina
 * @author Florent Benoit . Integration JOnAS/Tomcat
 */
final class CreateLoaderRule extends Rule {

    /**
     * Name of the attribute
     */
    private String attributeName;

    /**
     * Class name
     */
    private String loaderClass;


    /**
     * Create the rule (constructor)
     * @param digester digester to use
     * @param loaderClass class name
     * @param attributeName name of the attribute
     */
    public CreateLoaderRule(Digester digester, String loaderClass,
                            String attributeName) {
        super(digester);
        this.loaderClass = loaderClass;
        this.attributeName = attributeName;

    }

    /**
     * Start the rule
     * @param attributes attributes to use
     * @throws Exception if it can't start the rule
     */
    public void begin(Attributes attributes) throws Exception {

        // Look up the required parent class loader
        Container container = (Container) digester.peek();
        ClassLoader parentClassLoader = container.getParentClassLoader();

        // Instantiate a new Loader implementation object
        String className = loaderClass;
        if (attributeName != null) {
            String value = attributes.getValue(attributeName);
            if (value != null) {
                className = value;
            }
        }
        Class clazz = Class.forName(className);
        Class types[] = {ClassLoader.class };
        Object args[] = {parentClassLoader };
        Constructor constructor = clazz.getDeclaredConstructor(types);
        Loader loader = (Loader) constructor.newInstance(args);

        // Push the new loader onto the stack
        digester.push(loader);
        if (digester.getDebug() >= 1) {
            digester.log("new " + loader.getClass().getName());
        }

    }

    /**
     * End the rule
     * @throws Exception if it can't end the rule
     */
    public void end() throws Exception {

        Loader loader = (Loader) digester.pop();
        if (digester.getDebug() >= 1) {
            digester.log("pop " + loader.getClass().getName());
        }

    }


}
