/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Philippe Coq
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.web.catalina41;

//import JOnAS
import org.objectweb.jonas.common.Log;

//import objectweb
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

//import catalina
import org.apache.catalina.LifecycleException;
import org.apache.catalina.core.StandardContext;

/**
 * Define a JOnAS context. It is used to check if a context has been
 * defined in server.xml and to use this context while deploying the war
 * instead of creating a new one
 * @author Philippe Coq
 * @author Florent Benoit : Changes for Tomcat 4.1.x.
 */

public final class JOnASStandardContext extends StandardContext {
 
    /**
     * Logger for this service.
     */
    private static Logger logger = null;


    /**
     * Instance of Catalina 
     */
    private CatalinaJWebContainerServiceImpl catalinaService = null;

    /**
     * Constructor of the Context
     */
    public JOnASStandardContext() {
        super();
        logger = Log.getLogger(Log.JONAS_WEB_PREFIX);
    }

    /**
     * Set the Catalina WebContainer service
     * @param catalinaService the Catalina WebContainer service
     */
    public void setCatalinaJWebContainerService(CatalinaJWebContainerServiceImpl catalinaService) {
        this.catalinaService = catalinaService;
    }

    /**
     * Start the JOnAS context
     * @throws LifecycleException if the context can't be started
     */
    public synchronized void start() throws LifecycleException {
        logger.log(BasicLevel.DEBUG, " JOnASStandardContext start");
        if (catalinaService.isStarted()) {
            super.start();
        } 
    
    }
}
