/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.web.catalina41;

//import java
import java.io.File;
import java.net.URLClassLoader;
import java.net.URL;

//import javax
import javax.servlet.ServletContext;

//import catalina
import org.apache.catalina.Container;
import org.apache.catalina.Context;
import org.apache.catalina.Globals;
import org.apache.catalina.LifecycleException;
import org.apache.catalina.loader.WebappLoader;

/**
 * This class extends the WebappLoader of Catalina in order to define
 * our own setClasspath() method for JSP compiling.
 * As it's a private method in the super class, we overide the call to this method too.
 * More explanations can be found in the org.apache.catalina.loader.WebappLoader class
 *
 * @author Florent Benoit
 */

public class JWebappLoader extends WebappLoader {

    /**
     * Has this component been started ?
     */
    private boolean started = false;


    /**
     * Construct a new JWebappLoader with the specified class loader
     * to be defined as the parent of the ClassLoader we ultimately create.
     * @param parent The parent class loader
     */
    public JWebappLoader(ClassLoader parent) {
        super(parent);
    }

    /**
     * Set the appropriate context attribute for our class path.  This
     * is required only because Jasper depends on it.
     */
    private void setClassPath() {

        //get the container of the super class
        Container container = getContainer();

        if (!(container instanceof Context)) {
            return;
        }
        ServletContext servletContext = ((Context) container).getServletContext();
        if (servletContext == null) {
            return;
        }

        StringBuffer classpath = new StringBuffer();

        // Assemble the class path information from our class loader chain
        ClassLoader loader = getClassLoader();
        int n = 0;
        while ((loader != null)) {
            if (!(loader instanceof URLClassLoader)) {
                break;
            }
            URL repositories[] = ((URLClassLoader) loader).getURLs();
            for (int i = 0; i < repositories.length; i++) {
                String repository = repositories[i].toString();
                if (repository.startsWith("file://")) {
                    repository = repository.substring(7);
                } else if (repository.startsWith("file:")) {
                    repository = repository.substring(5);
                } else if (repository.startsWith("jndi:")) {
                    repository = servletContext.getRealPath(repository.substring(5));
                } else {
                    continue;
                }
                if (repository == null) {
                    continue;
                }
                if (n > 0) {
                    classpath.append(File.pathSeparator);
                }
                classpath.append(repository);
                n++;
            }
            loader = loader.getParent();
        }

        // Store the assembled class path as a servlet context attribute
        servletContext.setAttribute(Globals.CLASS_PATH_ATTR, classpath.toString());

    }

    /**
     * Start this component, initializing our associated class loader
     * and set the classpath with our own method.
     * @exception LifecycleException if a lifecycle error occurs
     */
    public void start() throws LifecycleException {
        super.start();
        started = true;
        setClassPath();
    }

    /**
     * Stop this component, finalizing our associated class loader.
     * @exception LifecycleException if a lifecycle error occurs
     */
    public void stop() throws LifecycleException {
        super.stop();
        started = false;
    }

    /**
     * Add a new repository to the set of repositories for this class loader.
     * @param repository Repository to be added
     */
    public void addRepository(String repository) {
        super.addRepository(repository);

        //call our method only if the loader is started
        if (started) {
            setClassPath();
        }

    }

}
