/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Hashtable;

/**
 * This class manages SessionContext objects for Session Stateful beans
 * It is responsible for keeping Contexts in memory and storing them
 * on disk when the swapper ask for more available memory.
 * @author Philippe Durieux
 */
public class JStatefulPool {
    private Hashtable memoryList = new Hashtable();
    private Hashtable diskList = new Hashtable();
    private int sessionCount = 0;

    /**
     * Constructor
     */
    public JStatefulPool() {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");
    }

    /**
     * add a new Context in the pool
     * @param ctx The Stateful Session Context
     * @return The Ident associated to it.
     */
    public synchronized int addContext(JStatefulContext ctx) {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");

        // sanity check
        if (ctx == null) {
            TraceEjb.logger.log(BasicLevel.ERROR, "null");
        }

        // First get a unique identifier
        int ctxid = sessionCount++;

        // Put in in the memoryList
        memoryList.put(new Integer(ctxid), (Object)ctx);

        // return Id
        return ctxid;
    }

    /**
     * get a Context matching this Id
     * @param ctxid Ident for this Session
     * @return The Session Context
     */
    public synchronized JStatefulContext getContext(int ctxid) {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");

        JStatefulContext ctx = (JStatefulContext) memoryList.get(new Integer(ctxid));
        // TODO

        // sanity check
        if (ctx == null) {
            TraceEjb.logger.log(BasicLevel.ERROR, "null");
        }

        return ctx;
    }

    /**
     * remove the Context matching this Id
     * @param ctxid Ident for this Session
     */
    public synchronized void removeContext(int ctxid) {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");

        memoryList.remove(new Integer(ctxid));
    }
}
