/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;

/**
 * For message logging
 * @author Sebastien Chassande-Barrioz sebastien.chassande@inrialpes.fr
 */


class DummyLogger extends LoggerImpl {

    /** 
     * Dummy Logger used when jonas_tm classes are used on client side
     */
    public void log(int level, java.lang.Object o) {
    }
}


public class TraceEjb {

    static public Logger logger = new DummyLogger();
    static public Logger query = new DummyLogger();
    static public Logger interp = new DummyLogger();
    static public Logger genclass = new DummyLogger();
    static public Logger coherence = new DummyLogger();
    static public Logger thread = new DummyLogger();
    static public Logger synchro = new DummyLogger();
    static public Logger swapper = new DummyLogger();
    static public Logger security = new DummyLogger();
    static public Logger mdb = new DummyLogger();
    static public Logger tx = new DummyLogger();
    static public Logger dd = new DummyLogger();
    static public Logger loaderlog = new DummyLogger();
    static public final String prefix = "org.objectweb.jonas_ejb";
    static public LoggerFactory loggerFactory = null;

    /**
     * Configure loggers for monolog
     * @param lf the logger factory
     */
    static public void configure(LoggerFactory lf) {
        loggerFactory = lf;
        logger = lf.getLogger(prefix);
        interp = lf.getLogger(prefix + ".interp");
        query = lf.getLogger(prefix + ".query");
        genclass = lf.getLogger(prefix + ".genclass");
        coherence = lf.getLogger(prefix + ".coherence");
        thread = lf.getLogger(prefix + ".thread");
        synchro = lf.getLogger(prefix + ".synchro");
        swapper = lf.getLogger(prefix + ".swapper");
        security = lf.getLogger(prefix + ".security");
        mdb = lf.getLogger(prefix + ".mdb");
        tx = lf.getLogger(prefix + ".tx");
        dd = lf.getLogger(prefix + ".dd");
        loaderlog = lf.getLogger("org.objectweb.jonas.loader");
    }

    static public boolean isVerbose() {
        return (logger != null) && logger.isLoggable(BasicLevel.WARN);
    }
    static public boolean isDebugThread() {
        return (thread != null) && thread.isLoggable(BasicLevel.DEBUG);
    }
    static public boolean isDebugSynchro() {
        return (synchro != null) && synchro.isLoggable(BasicLevel.DEBUG);
    }
    static public boolean isDebugSwapper() {
        return (swapper != null) && swapper.isLoggable(BasicLevel.DEBUG);
    }
    static public boolean isDebugIc() {
        return (interp != null) && interp.isLoggable(BasicLevel.DEBUG);
    }
    static public boolean isDebugSecurity() {
        return (security != null) && security.isLoggable(BasicLevel.DEBUG);
    }
    static public boolean isDebugJms() {
        return (mdb != null) && mdb.isLoggable(BasicLevel.DEBUG);
    }
    static public boolean isDebugTx() {
        return (tx != null) && tx.isLoggable(BasicLevel.DEBUG);
    }
    static public boolean isDebugDd() {
        return (dd != null) && dd.isLoggable(BasicLevel.DEBUG);
    }
    static public boolean isDebugQuery() {
        return (query != null) && query.isLoggable(BasicLevel.DEBUG);
    }


}
