/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): 
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.container.jorm;

import org.objectweb.jonas_ejb.deployment.api.EntityDesc;
import org.objectweb.jonas_ejb.deployment.api.BeanDesc;
import org.objectweb.jonas_ejb.deployment.api.EntityCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.DeploymentCmp2Desc;
import org.objectweb.jonas_ejb.container.JContainer;
import org.objectweb.jonas_ejb.container.TraceEjb;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.runtime.Mapper;
import org.objectweb.jorm.runtime.MapperJCA;
import org.objectweb.jorm.runtime.MapperJDBC;
import org.objectweb.util.monolog.api.BasicLevel;

import javax.sql.DataSource;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class RdbFactory extends MedorFactory {

    public RdbFactory() {
        super();
    }

    public void init(EntityDesc dd, JContainer cont, String mapperName) {
        super.init(dd, cont, mapperName);
    }

    public Object getConnection(Object hints) throws PException {
        if (hints == null) {
            return mapper.getConnection();
        } else {
            return mapper.getConnection(hints);
        }
    }

    public void releaseConnection(Object conn) throws PException {
        mapper.closeConnection(conn);
    }

    protected void setMapper(String mapperName) throws PException {
        MapperManager mm = MapperManager.getInstance();
        synchronized (mm) {
            mapper = (Mapper) mm.getMapper(mapperName, cont);
            if (mapper == null) {
                // datasource has been found by its JNDI name in the bean DD
                if (datasource instanceof DataSource) {
                    mapper = new MapperJDBC();
                } else {
                    mapper = new MapperJCA();
                }
                mapper.setMapperName(mapperName);
                // mapping name is used to get the meta-information
                mapper.setMappingName(DeploymentCmp2Desc.PROJECT_NAME);
                mapper.setConnectionFactory(datasource);
                mapper.setMetaInfoManager(miManager);
                mm.addMapper((Mapper) mapper, cont);
            }
        }
    }


    //public String getMIMappingFactory(EntityDesc bd) {
    //    return "org.objectweb.jorm.metainfo.lib.rdb.BasicRDBMappingFactory";
    //}
}
