/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Sebastien Chassande
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.genic;

import org.objectweb.jonas_ejb.deployment.api.EjbRelationshipRoleDesc;
import org.objectweb.jonas_ejb.deployment.api.EntityCmp2Desc;
import org.objectweb.jonas_ejb.lib.BeanNaming;


/**
 * This class is used in the velocity contexts. It represents a CMR field.
 *
 * @author S.Chassande-Barrioz
 */
public class VcCMRField {

    // Becarefull the number are ordered and encoded on 3 bits
    // - bit 0: 0=>Unidirectional  /  1=>bidirectional
    // - bit 1: 0=>One  /  1=>Many
    // - bit 2: 0=>One  /  1=>Many

    private String name;
    private String uFLName;
    private String typeName;
    private EjbRelationshipRoleDesc rsr;
    private EjbRelationshipRoleDesc rsr2;
    private byte relationType = -1;

    /**
     * This field has a non null value if the cmr field is a multivalued
     * relation
     */
    private String genClassName = null;
    private EntityCmp2Desc element = null;

    /**
     * It builds a cmr field descriptor usable in velocity contexts.
     */
    public VcCMRField(EjbRelationshipRoleDesc rsr) throws GenICException {
        this.rsr = rsr;
        name = rsr.getCmrFieldName();
        uFLName = upperFL(name);
        element = rsr.getLinkedEntityCmp2Desc();
        if (rsr.isMultiValuedCMR()) {
            typeName = rsr.getCmrFieldType().getName();
            if (java.util.Collection.class.equals(rsr.getCmrFieldType())) {
                genClassName = "org.objectweb.jonas_ejb.container.jorm.Set";
            } else if (java.util.Set.class.equals(rsr.getCmrFieldType())) {
                genClassName = "org.objectweb.jonas_ejb.container.jorm.Set";
            } else {
                throw new GenICException("Unauthorized multivalued relation type:"
                                         + rsr.getCmrFieldType());
            }
        } else {
            Class c = rsr.getLinkedEntityCmp2Desc().getLocalClass();
            if (c == null) {
                c = rsr.getLinkedEntityCmp2Desc().getRemoteClass();
            }
            typeName = c.getName();
        }
        rsr2 = rsr.getOppositeEjbRelationshipRoleDesc();
        relationType = rsr.getRelationType();
    }

    /**
     * It returns the name of the cmr field.
     */
    public String getName() {
        return name;
    }

    /**
     * It returns the type name of the cmr field. It is the referenced class
     * if the relation is multiple, otherwise the type of the multivalued
     * relation (java.util.Collection or java.util.Set).
     */
    public String getTypeName() {
        return typeName;
    }

    /**
     * It returns the name of cmr field with the first letter capitalized.
     */
    public String getUFLName() {
        return uFLName;
    }

    /**
     * It returns the JOnAS meta object which represents the cmr field, a
     * EjbRelationshipRoleDesc.
     */
    public EjbRelationshipRoleDesc getRsr() {
        return rsr;
    }

    /**
     * It returns the JOnAS meta object which represents the opposite cmr field,
     * a EjbRelationshipRoleDesc.
     */
    public String getOppositeUFLCMRName() {
        return upperFL(rsr2.getCmrFieldName());
    }

    public String getHelperClassName() {
        return "JOnAS" + rsr.getEjbName() + "CoherenceHelper";
    }
    public String getHelperFQClassName() {
        String pn = BeanNaming.getPackageName(rsr2.getLinkedEntityCmp2Desc().getFullDerivedBeanName());

        return (pn != null && pn.length() > 0
                ? pn + "." + getHelperClassName()
                : getHelperClassName());
        //return rsr2.getLinkedEntityCmp2Desc().getFullDerivedBeanName() + "."
        //    + getHelperClassName();
    }

    public String getOppositeHelperClassName() {
        return "JOnAS" + rsr2.getEjbName() + "CoherenceHelper";
    }
    public String getOppositeHelperFQClassName() {
        String pn = BeanNaming.getPackageName(rsr.getLinkedEntityCmp2Desc().getFullDerivedBeanName());

        return (pn != null && pn.length() > 0
                ? pn + "." + getOppositeHelperClassName()
                : getOppositeHelperClassName());

        //return rsr.getLinkedEntityCmp2Desc().getFullDerivedBeanName() + "."
        //		+ getOppositeHelperClassName();
    }

    /**
     * If the cmr is multiple, it returns the class name of the gen class which
     * must be used, otherwise null. The class which the is returned implements
     * the interface which the name is returned by the 'getTypeName' method.
     */
    public String getGenClassName() {
        return genClassName;
    }

    public EntityCmp2Desc getElement() {
        return element;
    }

    /**
     * It capitallizes the first letter of a word.
     */
    public String upperFL(String word) {
        return Character.toUpperCase(word.charAt(0)) + word.substring(1);
    }

    public boolean isOOu() {
        return relationType == EjbRelationshipRoleDesc.OOU;
    }
    public boolean isOOb() {
        return relationType == EjbRelationshipRoleDesc.OOB;
    }
    public boolean isOMu() {
        return relationType == EjbRelationshipRoleDesc.OMU;
    }
    public boolean isMOu() {
        return relationType == EjbRelationshipRoleDesc.MOU;
    }
    public boolean isMOb() {
        return relationType == EjbRelationshipRoleDesc.MOB;
    }
    public boolean isOMb() {
        return relationType == EjbRelationshipRoleDesc.OMB;
    }
    public boolean isMMu() {
        return relationType == EjbRelationshipRoleDesc.MMU;
    }
    public boolean isMMb() {
        return relationType == EjbRelationshipRoleDesc.MMB;
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("\n    Name                 = "  + getName());
        ret.append("\n    TypeName             = " + getTypeName());
        ret.append("\n    UFLName              = " + getUFLName());
        ret.append("\n    OppositeUFLCMRName   = " + getOppositeUFLCMRName());
        ret.append("\n    HelperClassName      = " + getHelperClassName());
        ret.append("\n    HelperFQClassName    = " + getHelperFQClassName());
        ret.append("\n    OppositeHelperClassName   = " + getOppositeHelperClassName());
        ret.append("\n    OppositeHelperFQClassName = " + getOppositeHelperFQClassName());
        ret.append("\n    GenClassName         = " + getGenClassName());
        ret.append("\n    isOOu                = " + isOOu());
        ret.append("\n    isOOb                = " + isOOb());
        ret.append("\n    isOMu                = " + isOMu());
        ret.append("\n    isOMb                = " + isOMb());
        ret.append("\n    isMOb                = " + isMOb());
        ret.append("\n    isMNu                = " + isMMu());
        ret.append("\n    isMNb                = " + isMMb());
        return (ret.toString());
    }
}
