/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Helene Joanin
 * Contributor(s): ______________________________________.
 * 00/09/14 Santiago Gala (sgala@hisitech.com)
 *          Parameters can be followed by a number in the WHERE clause
 * JOnAS 2.2.1 Joe Gittings has proposed to code method signature for security
 *          in order to avoid same signature for inherited methods.
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.genic;


import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.StringTokenizer;
import java.util.Vector;

import org.objectweb.jonas_ejb.lib.BeanNaming;
import org.objectweb.jonas_ejb.lib.EjbqlQueryTreeHolder;
import org.objectweb.jonas_ejb.lib.JavaType;
import org.objectweb.jonas_ejb.deployment.api.MethodDesc;
import org.objectweb.jonas_ejb.deployment.api.BeanDesc;
import org.objectweb.jonas_ejb.deployment.api.EntityCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.EntityJdbcCmp1Desc;
import org.objectweb.jonas_ejb.deployment.api.FieldJdbcDesc;
import org.objectweb.jonas_ejb.deployment.api.MethodJdbcCmp1Desc;
import org.objectweb.jonas_ejb.deployment.api.EntityJdbcCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.MethodCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.DeploymentDescException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.api.MedorException;
import org.objectweb.jorm.type.api.PType;


/**
 * This class is the "Velocity context" for a interface method or a bean method
 * used in the Velocity Templates.
 * @author: Helene Joanin (Bull)
 */

public class VcMethod {

    private String  mName = null;
    private String  mCapName = null;
    private StringBuffer mFormalParameters = null;
    private StringBuffer mActualParameters = null;
    private StringBuffer mExceptionList = null;
    private String  mSecuritySignature;
    private int     mTxAttribute = MethodDesc.TX_NULL;
    private String  mReturnType;
    private boolean isFinderByPk = false;
    private boolean isFinderAll = false;
    private boolean isFinderSimple = false;
    private boolean isFinderEnum = false;
    private boolean isFinderCol = false;
    private boolean isEjbSelectSimple = false;
    private boolean isEjbSelectSet = false;
    private boolean isEjbSelectCol = false;
    private boolean isEjbSelectReturnRemoteBean = false;
    private boolean isEjbSelectReturnLocalBean = false;
    private boolean isCreate = false;
    private boolean isEjbCreate = false;
    private boolean isRemoveThis = false;
    private boolean isRemovePk = false;
    private boolean isRemoveHandle = false;
    private boolean isEjbRemove = false;
    private boolean isEjbLoad = false;
    private boolean isEjbStore = false;
    private boolean isEjbSetEntityContext = false;
    private boolean isEjbActivate = false;
    private String  mSqlStmt = null;
    private Vector  mParamWhereList = null;
    private Method  mMethod = null;    // Just for explicit error message.
    private int  methodIndex = -1;
    private Class[] parameterTypes = null;
    private BeanDesc beanDesc = null;
    private org.objectweb.medor.api.Field resEjbSelectField = null;
    private EntityCmp2Desc resEjbSelectResultBD = null;
    private Vector  mParamList = null;    // Only used in CMP2 for finder and ejbSelect methods

    VcMethod(Method method, MethodDesc methodDesc, BeanDesc dd) {
        if (methodDesc != null) {
            methodIndex = methodDesc.getIndex();
        }
        beanDesc = dd;
        /*
         * The 'method' may be
         * - a home interface's method
         * - a local home interface's method
         * - a remote interface's method
         * - a local interface's method
         * - a bean's method as
         * - a beans's ejbSelect() method,
         *   setEntityContext(), ejbActivate(), ejbCreate(), ejbLoad(), ejbStore(), ejbRemove().
         * - a beans's ejbSelect() method,
         * In the last case, the 'methodDesc' is null in case of
         * setEntityContext(), ejbActivate(), ejbLoad() and ejbStore().
         */

        Class declClass = method.getDeclaringClass();
        Class params[] = method.getParameterTypes();
        parameterTypes = params;
        Class exceptions[] = method.getExceptionTypes();
        mMethod = method;

        // Name and CapName
        mName = method.getName();
        mCapName = BeanNaming.firstToUpperCase(mName);

        // FormalParameters and ActualParameters
        mFormalParameters = new StringBuffer();
        mActualParameters = new StringBuffer();
        for (int p = 1; p <= params.length; p++) {
            mFormalParameters.append(JavaType.getName(params[p - 1]) + " " + "p" + p);
            mActualParameters.append("p" + p);
            if (p < params.length) {
                mFormalParameters.append(", ");
                mActualParameters.append(", ");
            }
        }

        // ExceptionList
        mExceptionList = new StringBuffer();
        for (int e = 1; e <= exceptions.length; e++) {
            mExceptionList.append(exceptions[e - 1].getName());
            if (e < exceptions.length) {
                mExceptionList.append(", ");
            }
        }

        // SecuritySignature
        // No methodDesc for bean's methods as ejbLoad(), ejbStore(), ...
        if (methodDesc != null) {
            mSecuritySignature = "";
            if (methodDesc.getRoleName().length > 0) {
                mSecuritySignature = BeanNaming.getSignature(dd.getEjbName(), method);
            }
        }

        // TxAttribute
        // No methodDesc for bean's methods as ejbLoad(), ejbStore(), ...
        if (methodDesc != null) {
            mTxAttribute = methodDesc.getTxAttribute();
        }

        // ReturnType
        mReturnType = JavaType.getName(method.getReturnType());

        // isFinderXXX
        boolean isFinder = false;
        // TODO: How to known a finder method is really a 'finder-all' method ?
        isFinderAll = false;

        if (mName.startsWith("find")
            && (javax.ejb.EJBHome.class.isAssignableFrom(declClass)
                || javax.ejb.EJBLocalHome.class.isAssignableFrom(declClass))
            ) {
            isFinder = true;
        }
        if (isFinder) {
            isFinderByPk = "findByPrimaryKey".equals(mName);
            if (java.util.Enumeration.class.equals(method.getReturnType())) {
                isFinderEnum = true;
            } else if (java.util.Collection.class.equals(method.getReturnType())) {
                isFinderCol = true;
            } else {
                isFinderSimple = true;
            }
        }

        // isCreate
        if (mName.startsWith("create")
            && (javax.ejb.EJBHome.class.isAssignableFrom(declClass)
                || javax.ejb.EJBLocalHome.class.isAssignableFrom(declClass))
            ) {
            isCreate = true;
        }

        // isEjbCreate
        if (mName.startsWith("ejbCreate")
            && javax.ejb.EnterpriseBean.class.isAssignableFrom(declClass)) {
            isEjbCreate = true;
        }

        // isRemoveXXX
        boolean isRemove = "remove".equals(mName);
        if (isRemove) {
            if (declClass.equals(javax.ejb.EJBObject.class)
                || declClass.equals(javax.ejb.EJBLocalObject.class)) {
                isRemoveThis = true;
            } else if (declClass.equals(javax.ejb.EJBHome.class)
                       || declClass.equals(javax.ejb.EJBLocalHome.class)) {
                if (params[0].equals(javax.ejb.Handle.class)) {
                    isRemoveHandle = true;
                } else {
                    isRemovePk = true;
                }
            }
        }

        // isEjbRemove, isEjbLoad, isEjbStore
        if (javax.ejb.EnterpriseBean.class.isAssignableFrom(declClass)
            && (params.length == 0)) {
            if ("ejbRemove".equals(mName)) {
                isEjbRemove = true;
            } else if ("ejbLoad".equals(mName)) {
                isEjbLoad = true;
            } else if ("ejbStore".equals(mName)) {
                isEjbStore = true;
            }
        }

        // isEjbSetEntityContext
        if (javax.ejb.EnterpriseBean.class.isAssignableFrom(declClass)) {
            if ("setEntityContext".equals(mName)
                && (params.length == 1)
                && javax.ejb.EntityContext.class.equals(params[0])) {
                isEjbSetEntityContext = true;
            }
        }

        // isEjbActivate
        if (javax.ejb.EnterpriseBean.class.isAssignableFrom(declClass)) {
            if ("ejbActivate".equals(mName)
                && (params.length == 0)) {
                isEjbActivate = true;
            }
        }

        if (dd instanceof EntityJdbcCmp1Desc) {
            // For Entity CMP 1
            // SqlStmt, and ParamsWhere in case of FinderSimple, FinderEnum or FinderCol.
            EntityJdbcCmp1Desc edd = (EntityJdbcCmp1Desc) dd;
            Field[] fieldsBean = dd.getEjbClass().getFields();
            StringBuffer lf = new StringBuffer();
            StringBuffer lv = new StringBuffer();
            boolean firstLf = true;
            boolean firstLv = true;
            if (isEjbCreate) {
                for (int i = 0; i < fieldsBean.length; i++) {
                    Field f = fieldsBean[i];
                    if (edd.hasCmpFieldDesc(f)) {
                        FieldJdbcDesc fd = (FieldJdbcDesc) edd.getCmpFieldDesc(f);
                        if (firstLv) {
                            firstLv = false;
                        } else {
                            lf.append(", ");
                            lv.append(", ");
                        }
                        lf.append(fd.getJdbcFieldName());
                        lv.append("?");
                    }
                }
                mSqlStmt =
                    "insert into " + edd.getJdbcTableName()
                    + " (" + lf + ") values (" + lv + ")";
            } else if (isFinderByPk) {
                for (int i = 0; i < fieldsBean.length; i++) {
                    Field f = fieldsBean[i];
                    if (edd.hasCmpFieldDesc(f)) {
                        FieldJdbcDesc fd = (FieldJdbcDesc) edd.getCmpFieldDesc(f);
                        if (fd.isPrimaryKey()) {
                            if (firstLv) {
                                firstLv = false;
                            } else {
                                lf.append(", ");
                                lv.append(" and ");
                            }
                            lf.append(fd.getJdbcFieldName());
                            lv.append(fd.getJdbcFieldName() + "=?");
                        }
                    }
                }
                mSqlStmt =
                    "select " + lf + " from " + edd.getJdbcTableName()
                    + " where " + lv;
            } else if (isFinderSimple || isFinderEnum || isFinderCol) {
                if (!((MethodJdbcCmp1Desc) methodDesc).hasWhereClause()) {
                    throw new Error("No WHERE clause defined for the finder method '"
                                    + method.toString() + "'");
                }
                Vector posParams = new Vector();
                String clause_where = parseWhere (((MethodJdbcCmp1Desc) methodDesc).getWhereClause(), posParams);
                for (int i = 0; i < fieldsBean.length; i++) {
                    Field f = fieldsBean[i];
                    if (edd.hasCmpFieldDesc(f)) {
                        FieldJdbcDesc fd = (FieldJdbcDesc) edd.getCmpFieldDesc(f);
                        if (fd.isPrimaryKey()) {
                            if (firstLf) {
                                firstLf = false;
                            } else {
                                lf.append(", ");
                            }
                            lf.append(fd.getJdbcFieldName());
                        }
                    }
                }
                mSqlStmt =
                    "select " + lf + " from " + edd.getJdbcTableName()
                    + " " + clause_where;
                mParamWhereList = new Vector();
                for (int i = 0; i < posParams.size(); i++) {
                    int pos = ((Integer) posParams.elementAt(i)).intValue();
                    if (pos >= params.length) {
                        throw new Error("Wrong parameters number between the method definition and the WHERE clause of the method '" + method + "'");
                    }
                    VcParamWhere vpw = new VcParamWhere(params[pos], pos + 1);
                    mParamWhereList.addElement(vpw);
                }
            } else if (isEjbRemove) {
                for (int i = 0; i < fieldsBean.length; i++) {
                    Field f = fieldsBean[i];
                    if (edd.hasCmpFieldDesc(f)) {
                        FieldJdbcDesc fd = (FieldJdbcDesc) edd.getCmpFieldDesc(f);
                        if (fd.isPrimaryKey()) {
                            if (firstLv) {
                                firstLv = false;
                            } else {
                                lv.append(" and ");
                            }
                            lv.append(fd.getJdbcFieldName() + "=?");
                        }
                    }
                }
                mSqlStmt =
                    "delete from " + edd.getJdbcTableName()
                    + " where " + lv;
            } else if (isEjbLoad) {
                for (int i = 0; i < fieldsBean.length; i++) {
                    Field f = fieldsBean[i];
                    if (edd.hasCmpFieldDesc(f)) {
                        FieldJdbcDesc fd = (FieldJdbcDesc) edd.getCmpFieldDesc(f);
                        if (firstLf) {
                            firstLf = false;
                        } else {
                            lf.append(", ");
                        }
                        lf.append(fd.getJdbcFieldName());
                        if (fd.isPrimaryKey()) {
                            if (firstLv) {
                                firstLv = false;
                            } else {
                                lv.append(" and ");
                            }
                            lv.append(fd.getJdbcFieldName() + "=?");
                        }
                    }
                }
                mSqlStmt =
                    "select " + lf + " from " + edd.getJdbcTableName()
                    + " where " + lv;
            } else if (isEjbStore) {
                for (int i = 0; i < fieldsBean.length; i++) {
                    Field f = fieldsBean[i];
                    if (edd.hasCmpFieldDesc(f)) {
                        FieldJdbcDesc fd = (FieldJdbcDesc) edd.getCmpFieldDesc(f);
                        if (fd.isPrimaryKey()) {
                            if (firstLv) {
                                firstLv = false;
                            } else {
                                lv.append(" and ");
                            }
                            lv.append(fd.getJdbcFieldName() + "=?");
                        } else {
                            if (firstLf) {
                                firstLf = false;
                            } else {
                                lf.append(", ");
                            }
                            lf.append(fd.getJdbcFieldName() + "=?");
                        }
                    }
                }
                mSqlStmt =
                    "update " + edd.getJdbcTableName()
                    + " set " + lf + " where " + lv;
            }
        } else if (dd instanceof EntityJdbcCmp2Desc) {
            // ejbSelect() methods
            boolean isEjbSelect = false;
            if (mName.startsWith("ejbSelect")
                && javax.ejb.EntityBean.class.isAssignableFrom(declClass)) {
                isEjbSelect = true;
            }
            if (isEjbSelect) {
                if (java.util.Collection.class.equals(method.getReturnType())) {
                    isEjbSelectCol = true;
                } else if (java.util.Set.class.equals(method.getReturnType())) {
                    isEjbSelectSet = true;
                } else {
                    isEjbSelectSimple = true;
                }
                //
                try {
                    MethodCmp2Desc md = (MethodCmp2Desc) methodDesc;
                    resEjbSelectField = md.getQueryTreeHolder().getResField();
                    if (resEjbSelectField.getType().getTypeCode() == PType.TYPECODE_REFERENCE) {
                        if (((MethodCmp2Desc) methodDesc).isResultTypeMappingRemote()) {
                            isEjbSelectReturnRemoteBean = true;
                        } else {
                            isEjbSelectReturnLocalBean = true;
                        }
                        resEjbSelectResultBD = getBeanDescOfPath(splitPath(resEjbSelectField.getName()),
                                                                 (EntityCmp2Desc) dd,
                                                                 md.getQueryTreeHolder());
                    }
                } catch (DeploymentDescException e) {
                    throw new Error("Error while the EJB-QL parsing of '" + method.toString()
                                    + "': " + e.getMessage());
                }
            }
            if (isFinder || isEjbSelect) {
                mParamList = new Vector();
                for (int p = 0; p < params.length; p++) {
                    mParamList.add(new VcParam(params[p], dd));
                }
            }
            
        }

        // System.out.print("VcMethod: "+method.toString());
        // System.out.println(toString());
    }

    /**
     * Parses Strings with syntax "WHERE summary like ?<nnn>"
     * where <nnn> is a decimal integer that indexes (starting by 0)
     * the parameters in the findByXXX method
     * @author: Santiago Gala (sgala@hisitech.com)
     */
    private String parseWhere(String fragment, Vector pos) {
        StringTokenizer parseWhere = new StringTokenizer(fragment, "?\n", true);
        int numpos = pos.size();
        String new_where = "";
        String tok;
        while (parseWhere.hasMoreTokens()) {
            tok = parseWhere.nextToken();
            String numVar = "";
            if (Character.isDigit(tok.charAt(0))) {
                for (int i = 0; tok.length() > i && Character.isDigit(tok.charAt(i)); i++) {
                    numVar += tok.charAt(i);
                }
                if (numVar.length() > 0) {
                    new_where += tok.substring(numVar.length(), tok.length());
                    pos.setElementAt((Object) new Integer(Integer.parseInt(numVar) - 1), numpos - 1);
                }
            } else {
                new_where += tok;
                if (tok.charAt(0) == '?') {
                    pos.addElement(new Integer(numpos++));
                }
            }
        }
        return new_where;
    }

    public String getName() {
        return mName;
    }
    public String getCapName() {
        return mCapName;
    }
    public String getFormalParameters() {
        return mFormalParameters.toString();
    }
    public String getActualParameters() {
        return mActualParameters.toString();
    }
    public String getExceptionList() {
        return mExceptionList.toString();
    }
    public String getSecuritySignature() {
        if (mSecuritySignature == null) {
            throw new Error("VcMethod.getSecuritySignature() not available for the method '"
                            + mMethod.toString() + "'");
        }
        return mSecuritySignature;
    }
    public int getTxAttribute() {
        if (mTxAttribute == MethodDesc.TX_NULL) {
            throw new Error("VcMethod.getTxAttribute() not available for the method '"
                            + mMethod.toString() + "'");
        }
        return mTxAttribute;
    }
    public String getReturnType() {
        return mReturnType;
    }
    public boolean isFinderAll() {
        return isFinderAll;
    }
    public boolean isFinderByPk() {
        return isFinderByPk;
    }
    public boolean isFinderSimple() {
        return isFinderSimple;
    }
    public boolean isFinderEnum() {
        return isFinderEnum;
    }
    public boolean isFinderCol() {
        return isFinderCol;
    }
    public boolean isEjbSelectSimple() {
        return isEjbSelectSimple;
    }
    public boolean isEjbSelectSet() {
        return isEjbSelectSet;
    }
    public boolean isEjbSelectCol() {
        return isEjbSelectCol;
    }
    public boolean isEjbSelectReturnRemoteBean() {
        return isEjbSelectReturnRemoteBean;
    }
    public boolean isEjbSelectReturnLocalBean() {
        return isEjbSelectReturnLocalBean;
    }
    public boolean isCreate() {
        return isCreate;
    }
    public boolean isEjbCreate() {
        return isEjbCreate;
    }
    public boolean isRemoveThis() {
        return isRemoveThis;
    }
    public boolean isRemovePk() {
        return isRemovePk;
    }
    public boolean isRemoveHandle() {
        return isRemoveHandle;
    }
    public boolean isEjbRemove() {
        return isEjbRemove;
    }
    public boolean isEjbLoad() {
        return isEjbLoad;
    }
    public boolean isEjbStore() {
        return isEjbStore;
    }
    public boolean isEjbSetEntityContext() {
        return isEjbSetEntityContext;
    }
    public boolean isEjbActivate() {
        return isEjbActivate;
    }
    public String getSqlStmt() {
        if (mSqlStmt == null) {
            throw new Error("VcMethod.getSqlStmt() not available for the method '"
                            + mMethod.toString() + "'");
        }
        return mSqlStmt;
    }

    public Class[] getParameterTypes() {
        return parameterTypes;
    }
    public int getParametersNumber() {
        return parameterTypes.length;
    }

    public Vector getParamWhereList() {
        if (mParamWhereList == null) {
            throw new Error("VcMethod.getParamWhereList() not available for the method '"
                            + mMethod.toString() + "'");
        }
        return mParamWhereList;
    }

    public Vector getParamList() {
        if (mParamList == null) {
            throw new Error("VcMethod.getParamList() not available for the method '"
                            + mMethod.toString() + "'");
        }
        return mParamList;
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("\n    Name                  = " + getName());
        ret.append("\n    CapName               = " + getCapName());
        ret.append("\n    FormalParameters      = " + getFormalParameters());
        ret.append("\n    ActualParameters      = " + getActualParameters());
        ret.append("\n    ExceptionList         = " + getExceptionList());
        if (mSecuritySignature != null) {
            ret.append("\n    SecuritySignature     = " + getSecuritySignature());
        }
        if (mTxAttribute != MethodDesc.TX_NULL) {
            ret.append("\n    TxAttribute           = " + getTxAttribute());
        }
        ret.append("\n    ReturnType            = " + getReturnType());
        ret.append("\n    isFinderByPk          = " + isFinderByPk());
        ret.append("\n    isFinderSimple        = " + isFinderSimple());
        ret.append("\n    isFinderEnum          = " + isFinderEnum());
        ret.append("\n    isFinderCol           = " + isFinderCol());
        ret.append("\n    isCreate              = " + isCreate());
        ret.append("\n    isEjbCreate           = " + isEjbCreate());
        ret.append("\n    isRemoveThis          = " + isRemoveThis());
        ret.append("\n    isRemovePk            = " + isRemovePk());
        ret.append("\n    isRemoveHandle        = " + isRemoveHandle());
        ret.append("\n    isEjbRemove           = " + isEjbRemove());
        ret.append("\n    isEjbLoad             = " + isEjbLoad());
        ret.append("\n    isEjbStore            = " + isEjbStore());
        ret.append("\n    isEjbSetEntityContext = " + isEjbSetEntityContext());
        ret.append("\n    isEjbActivate         = " + isEjbActivate());
        ret.append("\n    isEjbSelectSimple     = " + isEjbSelectSimple());
        ret.append("\n    isEjbSelectSet        = " + isEjbSelectSet());
        ret.append("\n    isEjbSelectCol        = " + isEjbSelectCol());
        if (mSqlStmt != null) {
            ret.append("\n    SqlStmt               = " + getSqlStmt());
        }
        if (mParamWhereList != null) {
            ret.append("\n    ParamWhereList        = " + getParamWhereList());
        }
        return (ret.toString());
    }

    public int getMethodIndex() {
        return methodIndex;
    }

    /**
     * Return the Medor field of tuple result of an ejbSelect method
     */
    public org.objectweb.medor.api.Field getResEjbSelectField() {
        return resEjbSelectField;
    }

    /**
     * Return the bean descriptor associated to the result of an ejbSelect method
     * in case of the result is a bean.
     */
    public EntityCmp2Desc getResEjbSelectResultBD() {
        return resEjbSelectResultBD;
    }

    /*
     * Return the Medor get method name of the tuple result of an ejbSelect method
     */
    public String getTupleGetter() {
        PType type = resEjbSelectField.getType();
        switch(type.getTypeCode()) {
        case PType.TYPECODE_BOOLEAN:
            return "getBoolean";
        case PType.TYPECODE_BYTE:
            return "getByte";
        case PType.TYPECODE_CHAR:
            return "getChar";
        case PType.TYPECODE_DATE:
            return "getDate";
        case PType.TYPECODE_DOUBLE:
            return "getDouble";
        case PType.TYPECODE_FLOAT:
            return "getFloat";
        case PType.TYPECODE_INT:
            return "getInt";
        case PType.TYPECODE_LONG:
            return "getLong";
        case PType.TYPECODE_SERIALIZED:
            return "getObject";
        case PType.TYPECODE_SHORT:
            return "getShort";
        case PType.TYPECODE_STRING:
            return "getString";
        case PType.TYPECODE_BIGDECIMAL:
        case PType.TYPECODE_REFERENCE:
        case PType.TYPECODE_BYTEARRAY:
        case PType.TYPECODE_CHARARRAY:
        case PType.TYPECODE_OBJBOOLEAN:
        case PType.TYPECODE_OBJBYTE:
        case PType.TYPECODE_OBJCHAR:
        case PType.TYPECODE_OBJDOUBLE:
        case PType.TYPECODE_OBJFLOAT:
        case PType.TYPECODE_OBJINT:
        case PType.TYPECODE_OBJLONG:
        case PType.TYPECODE_OBJSHORT:
        default:
            return "getObject";
        }
    }


    /**
     * split a dot separated path into tokens
     */
    String[] splitPath(String path) {
        StringTokenizer st = new StringTokenizer(path, ".");
        String[] ret = new String[st.countTokens()];
        for (int i = 0; i < ret.length; i++) {
            ret[i] = st.nextToken();
        }
        return ret;
    }

    /**
     * Return the BeanDesc of the given path corresponding to a single_valued_navigation
     * ie identification_variable . [single_valued_cmr_field . ]* single_valued_cmr_field
     * The path may be ended by the _PNAME (Field.PNAMENAME).
     */
    EntityCmp2Desc getBeanDescOfPath(String []path,
                                     EntityCmp2Desc bdOfBegining,
                                     EjbqlQueryTreeHolder qth)
        throws DeploymentDescException {
        EntityCmp2Desc bdRes = bdOfBegining;
        String [] vDecl = qth.getName(path[0]);
        if (vDecl.length == 1) {
            // range variable declaration case : abstract_schema_name AS identifier
            bdRes = bdRes.getDeploymentDesc().getBeanDescWithSchemaName(vDecl[0]);
        } else {
            // Collection member declaration case :
            // IN (collection_valued_path_expression) AS identifier
            bdRes = getBeanDescOfPath(vDecl, bdRes, qth);
        }
        for (int i=1; i < path.length; i++) {
            // Ignore the "_PNAME" field
            if (!path[i].equals(org.objectweb.medor.api.Field.PNAMENAME)) {
                bdRes = bdRes.getEjbRelationshipRoleDesc(path[i]).getLinkedEntityCmp2Desc();
            }
        }
        return bdRes;
    }

}
