/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Helene Joanin
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.genic;


import org.objectweb.jonas_ejb.deployment.api.BeanDesc;
import org.objectweb.jonas_ejb.lib.JavaType;
import org.objectweb.jonas_ejb.lib.JormType;


/**
 * This class is the "Velocity context" for a parameter of a finder or select method
 * used in the Velocity Templates.
 */

public class VcParam {

    private String mTypeName = null;
    private String mJormTypeName = null;
    private boolean mIsEjbLocal = false;
    private String mEjbName = null;

    VcParam(Class type, BeanDesc dd) {
        mTypeName = JavaType.getName(type);
        mJormTypeName = JormType.getPTypeDef(type);
        mIsEjbLocal = javax.ejb.EJBLocalObject.class.isAssignableFrom(type);
        if (mIsEjbLocal) {
            BeanDesc bdp = dd.getDeploymentDesc().getBeanDescWithLocalInterface(mTypeName);
            if (bdp == null) {
                throw new Error("VcParam: Cannot get the BeanDesc associated to the interface local '" + mTypeName + "'");
            }
            mEjbName = bdp.getEjbName();
        }
    }

    public String getTypeName() {
        return mTypeName;
    }

    public String getJormType() {
        return mJormTypeName;
    }

    public boolean isEjbLocal() {
        return mIsEjbLocal;
    }

    public String getEjbName() {
        if (!isEjbLocal()) {
            throw new Error("VcParam.getEjbName(): No BeanDesc associated to the param '"
                            + mTypeName + "'");
        }
        return mEjbName;
    }

}

