/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Sebastien Chassande-Barrioz for INRIA
 * Contributor(s): Ph Durieux (Bull)
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.lib;

import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.rdb.RDBClassMapping;
import org.objectweb.jorm.metainfo.api.rdb.RDBValueMapping;
import org.objectweb.jorm.metainfo.api.rdb.RDBColumnSpec;
import org.objectweb.jonas_ejb.deployment.api.EntityCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.FieldDesc;
import org.objectweb.jonas_ejb.deployment.api.EntityJdbcCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.FieldJdbcDesc;
import org.objectweb.jonas_ejb.deployment.api.EjbRelationDesc;
import org.objectweb.jonas_ejb.deployment.api.EjbRelationJdbcDesc;
import org.objectweb.jonas_ejb.deployment.api.EjbRelationshipRoleDesc;

import java.util.Iterator;
import java.util.Map;
import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class is an implementation of the MappingBuilder interface for the rdb
 * mapper.
 * @author Sebastien Chassande-Barrioz
 */
public class RdbMappingBuilder implements MappingBuilder {

    static private Logger logger = null;

    public RdbMappingBuilder() {
	logger = Log.getLogger("org.objectweb.jonas.mapper");
        logger.log(BasicLevel.DEBUG, "constructor");
    }

    /**
     * This implementation is linked to the rdb mapper.
     */
    public final static String MAPPER_NAME = "rdb";

    public void createClassMapping(Class clazz,
				   EntityCmp2Desc bd,
				   String projectName) {

        RDBClassMapping cm = (RDBClassMapping) clazz.createClassMapping(MAPPER_NAME, projectName);

        // Table name = Abstract Shema Name
	// This name must be capitalized because of Oracle constraints when accessing metadata.
	// A '_' character is added to the table names in order to avoid to use
	// the key words of the database
	String tablename = bd.getAbstractSchemaName().toUpperCase()+"_";
        cm.setTableName(tablename);

        // Primary key definition = name of the field
        for (Iterator it = bd.getCmpFieldDescIterator(); it.hasNext();) {
            FieldDesc fd = (FieldDesc) it.next();
            if (fd.isPrimaryKey()) {
		String pkname = fd.getFieldName()+"_";
                cm.addPrimaryKey(pkname);
            }
        }
    }

    public void createValueMapping(PrimitiveElement pe,
				   FieldDesc desc,
				   String projectName) {

	RDBValueMapping vm = (RDBValueMapping) pe.createValueMapping(MAPPER_NAME, projectName);

	// Assign cmp-field name with default type.
	// A '_' character is added to the field names in order to avoid to use
	// the key words of the database
	String cmpfname = pe.getName()+"_";
        vm.createRDBColumnSpec(cmpfname, null, false);
    }

    public void createClassMapping(GenClassRef gcr,
				   EjbRelationshipRoleDesc rsrd,
				   String projectName) {

	RDBClassMapping cm = (RDBClassMapping) gcr.createClassMapping(MAPPER_NAME, projectName);

	// joint table name is build from both abstract schema names
	EjbRelationDesc rd = rsrd.getEjbRelationDesc();
	EjbRelationshipRoleDesc rsrd1 = rd.getRelationshipRoleDesc1();
	EjbRelationshipRoleDesc rsrd2 = rd.getRelationshipRoleDesc2();
	EntityCmp2Desc ed1 = rsrd1.getLinkedEntityCmp2Desc();
	EntityCmp2Desc ed2 = rsrd2.getLinkedEntityCmp2Desc();
	String jointbl = ed1.getAbstractSchemaName().toUpperCase() + "_" + ed2.getAbstractSchemaName().toUpperCase();
	cm.setTableName(jointbl);

	//TODO: fill with foreign key
    }

    public void createValueMapping(PrimitiveElement pe,
                                   FieldDesc desc,
                                   EjbRelationshipRoleDesc rsrd,
                                   byte type,
                                   String projectName) {

	RDBValueMapping vm = (RDBValueMapping) pe.createValueMapping(MAPPER_NAME, projectName);

	EntityCmp2Desc ed1 = rsrd.getLinkedEntityCmp2Desc();
	String cmrfname = pe.getName();
	vm.createRDBColumnSpec(cmrfname, null, false);
    }
}
