/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.objectweb.jonas.jtests.clients.entity;

import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.beans.ejbql.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * This is an advanced test suite for home interface on entity bean EJBQL.
 */
public class F_EjbqlEC2 extends A_Cmp2Util {

    private static String BEAN_HOME_CUSTOMER = "CustomerHomeRemote";
    private static String BEAN_HOME_RESERVATION = "ReservationHomeRemote";
    private static String BEAN_HOME_CRUISE = "CruiseHomeRemote";
    private static String BEAN_HOME_SHIP = "ShipHomeRemote";
    private static String BEAN_HOME_CABIN = "CabinHomeRemote";
    private static String BEAN_HOME_ADDRESS = "AddressHomeRemote";

    protected static CustomerHomeRemote customerhome = null;
    protected static ReservationHomeRemote reservationhome = null;
    protected static CruiseHomeRemote cruisehome = null;
    protected static ShipHomeRemote shiphome = null;
    protected static CabinHomeRemote cabinhome = null;
    protected static AddressHomeRemote addresshome = null;
    protected static CreditCardHomeRemote creditcardhome = null;

    public F_EjbqlEC2(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
        boolean ok = false;
        int nbtry = 0;
        while (!ok && nbtry < 3) {
            if (!isInit) {
                // load bean if not loaded yet
                useBeans("ejbql", false);
                // lookup home used in the tests
	      
                try {
                    customerhome  = (CustomerHomeRemote)
                        PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_CUSTOMER),
                                                    CustomerHomeRemote.class);
                    shiphome  = (ShipHomeRemote)
                        PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_SHIP),
                                                    ShipHomeRemote.class);
                    cruisehome = (CruiseHomeRemote)
                        PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_CRUISE),
                                                    CruiseHomeRemote.class);
                    reservationhome = (ReservationHomeRemote)
                        PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_RESERVATION),
                                                    ReservationHomeRemote.class);
                    cabinhome = (CabinHomeRemote)
                        PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_CABIN),
                                                    CabinHomeRemote.class);  
                    addresshome = (AddressHomeRemote)
                        PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_ADDRESS),
                                                    AddressHomeRemote.class);  
                } catch (NamingException e) {
                    fail("Cannot get bean home: " + e.getMessage());
                }
                // check if tables have been initialized
                try {
                    customerhome.findByPrimaryKey(new Integer(80));
                } catch (Exception e) {
                    // Make the initialization needed for the tests
                    try {
                        utx.begin(); 
                        String cities[] = new String[6];
                        cities[0] = "Minneapolis"; cities[1] = "St. Paul"; cities[2] = "Rochester";
                        cities[3] = "Winona"; cities[4] = "Wayzata"; cities[5] = "Eagan";
                        ShipRemote shipA = null;
                        CruiseRemote cruiseA = null;
                        shipA = shiphome.create(new Integer(10772), "Ship A", 30000.0); 
                        cruiseA = cruisehome.create("Cruise A", new Integer(10772)); 
                        for (int jj = 1; jj <= 10; jj++) {
                            ShipRemote ship = shiphome.create(new Integer(jj), "Ship " + jj, 30000.0 + (10000.0 * jj));
                        }
                        cruiseA = null;
                        CruiseRemote cruiseB = null;
                        ShipRemote ship1 = shiphome.findByPrimaryKey(new Integer(1));
                        cruiseA = cruisehome.create("Alaska Cruise", new Integer(1)); 
                        cruiseB = cruisehome.create("Bohemian Cruise", new Integer(1));
                        for (int kk = 80; kk <= 99; kk++) {
                            CustomerRemote customer = null;

                            customer = customerhome.create(new Integer(kk));
                            customer.setName(new Name("Smith"+kk,"John") );
                            customer.addPhoneNumber("612-555-12"+kk,(byte) 1);

                            customer.setAddress("10"+kk+" Elm Street", 
                                                cities[(kk-80)%6], 
                                                (kk%2==0?"MN":"CA"), 
                                                "5540"+(kk%5+1));
                            customer.setHasGoodCredit((kk%4 == 0));
                            if (kk%4 == 0)
                                customer.setCreditCard(new Date(System.currentTimeMillis()+100000), "100001"+kk, "MasterCard", "CA" ,"CETELEM", "rue de jeunet", cities[0], "France",  "3888" );
                        }
                        // Creating Customers 1-6, each with 2 reservations for 2 cabins
                        Calendar date = Calendar.getInstance();
                        date.set(2002,10,1);

                        for (int kk=201; kk<207; kk++) {
                            Collection customers = new ArrayList();
                            CustomerRemote cust = customerhome.create(new Integer(kk));
                            cust.setName(new Name("Customer "+kk,"Mike"));
                            cust.setHasGoodCredit( (kk%2==0) );  // odds are bums
                            cust.setAddress("50"+kk+" Main Street","Minneapolis","MN","5510"+kk);
                            customers.add(new Integer(kk));  // put this single customer in the collection

                            Collection reservations = new ArrayList();

                            for (int jj=0; jj<2; jj++) {

                                ReservationRemote reservation = reservationhome.create(cruiseA.getId(), customers);
                                reservation.setDate(date.getTime());
                                reservation.setAmountPaid(1000*kk+100*jj+2000);
		    
                                date.add(Calendar.DAY_OF_MONTH, 7);

                                Set cabins = new HashSet();
                                CabinRemote cabin = cabinhome.create(new Integer(1000+kk*100+jj));
                                cabin.setDeckLevel(kk-200);
                                cabin.setName("Cabin "+kk+"0"+jj+"1");

                                cabins.add(cabin.getId());
                                cabin = cabinhome.create(new Integer(1000+kk*100+10+jj));
                                cabin.setDeckLevel(kk-200);
                                cabin.setName("Cabin "+kk+"0"+jj+"2");

                                cabins.add(cabin.getId());

                                reservation.setAllCabins(cabins);  // this reservation has 2 cabins

                            }
                        }
                        String fnames[] = new String[5];
                        fnames[0]="John"; fnames[1]="Paul"; fnames[2]="Ringo";
                        fnames[3]="Joe"; fnames[4]="Roger";

                        String lnames[] = new String[3];
                        lnames[0]="Smith"; lnames[1]="Johnson"; lnames[2]="Star";
                        // Creating Customers 50-69
                        for (int kk=50; kk<=69; kk++) {
                            CustomerRemote customer = customerhome.create(new Integer(kk));
                            customer.setName( new Name(lnames[(kk-50)%3], fnames[(kk-50)%5]) );
                            customer.addPhoneNumber("612-555-12"+kk,(byte)1);
                            customer.setAddress("10"+kk+" Elm Street", 
                                                cities[(kk-50)%6], 
                                                (kk%2==0?"MN":"CA"), 
                                                "5540"+(kk%5+1));
                            customer.setHasGoodCredit((kk%4==0));

                            // Some customers will have reservations already on one of the two cruises..
                            if (kk%3!=0) {
                                Collection customers = new ArrayList();
                                customers.add(customer.getId());  // put this single customer in the collection
                                ReservationRemote reservation = reservationhome.create((kk%3==1?cruiseA.getId():cruiseB.getId()), customers);
                                reservation.setDate(date.getTime());
                                reservation.setAmountPaid(10*kk+2000);
                                date.add(Calendar.DAY_OF_MONTH, 7);
                            }
                        }

                        // Creating Customers 100-109

                        for (int kk=100; kk<=109; kk++) {
                            CustomerRemote customer = customerhome.create(new Integer(kk));
                            customer.setName( new Name("Lennon"+kk, "Paul") );
                            customer.addPhoneNumber("666-543-12"+kk,(byte)1);
							      
                            customer.setAddress("10"+kk+" Abbey Road", 
                                                cities[(kk-100)%6], 
                                                (kk%2==0?"FL":"WA"), 
                                                "5540"+(kk%5+1));
                            customer.setHasGoodCredit((kk%4==0));
                        }

                    } catch (Exception i) {
                        i.printStackTrace(System.out);
                        fail("InitialState creation problem: "+i);
                    } finally {
                        try {
                            utx.commit();
                        } catch (Exception ii) {
                        }
                    }
                }
                isInit = true;
            }
            // Check that all is OK. Sometimes, a test has failed and has corrupted
            // the bean state in the database. We must unload and reload the bean then.
            nbtry++;
            try {
                if (initStateOK()) {
                    ok = true;
                }
            } catch (Exception e) {
                e.printStackTrace(System.out);
            }
            if (!ok) {
                isInit = false;
                unloadBeans("ejbql");
            }
        }
    }


    /*
     * Check that we are in the same state as after the tables creation for thoses beans A and B
     * (ie if it is the initial state)
     */
    boolean initStateOK() throws Exception {
        // just verify that all ejb are correcly created. Nothing about relation here because we testing only ejbql
        boolean isOk = true;

        msgerror = new StringBuffer();

        ShipRemote ship10772 = shiphome.findByPrimaryKey(new Integer(10772));
        for (int i=1; i<=10; i++) {
            ShipRemote ship = shiphome.findByPrimaryKey(new Integer(i));
        }

        CruiseRemote cruiseA = cruisehome.findByName("Cruise A");
        CruiseRemote cruiseB = cruisehome.findByName("Alaska Cruise");
        CruiseRemote cruiseC = cruisehome.findByName("Bohemian Cruise");
        for (int j=10; j<=12; j++) {
            CruiseRemote cruise = cruisehome.findByPrimaryKey(new Integer(j));
        }
 
        for (int k=80; k<=109; k++) {
            CustomerRemote customer = customerhome.findByPrimaryKey(new Integer(k));
        }

        for (int l=201; l<=206; l++) {
            CustomerRemote customer1 = customerhome.findByPrimaryKey(new Integer(l));
        } 
                           
        for (int p=50; p<=69; p++) {
            CustomerRemote customer2 = customerhome.findByPrimaryKey(new Integer(p));
        } 
        for (int kk=201;kk<207;kk++) {
            for (int jj=0;jj<2;jj++) {
                CabinRemote cabin = cabinhome.findByPrimaryKey(new Integer(1000+kk*100+jj));
                CabinRemote cabin1 = cabinhome.findByPrimaryKey(new Integer(1000+kk*100+10+jj));
            }
        }

        for (int kkh=10;kkh<=34;kkh++) {
            ReservationRemote reservation = reservationhome.findByPrimaryKey(new Integer(kkh));
        } 
        return isOk;
    }

    /**
     * 	Finding Customer having name 'John Smith85'
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.lastName = ?1 AND c.firstName = ?2
     */
    public void testAnd1() throws Exception {
        CustomerRemote customer85 = customerhome.findByExactName("Smith85","John");
        assertEquals("Wrong Customer : ",new Integer(85), customer85.getId());
        // checkIsInitialState(); not necessary in this test
    }

    /**
     * 	Finding Customer 'Smith90'
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.lastName = 'Smith90'
     */
    public void testEgalString() throws Exception {
        CustomerRemote customer90 = customerhome.findSmith90();
        assertEquals("Wrong Customer : ",new Integer(90), customer90.getId());
        // checkIsInitialState(); not necessary in this test
    }

    /**
     * 	Finding Customers having GoodCredit
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.hasGoodCredit = TRUE 
     */
    public void testEgalBoolean() throws Exception {
        Collection mplscustomers = customerhome.findByGoodCredit();
        ArrayList customers_id  = new ArrayList();
        Iterator customer = mplscustomers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();

        for (int ii=80;ii<=96;ii=ii+4) {
            result.add(new Integer(ii));
        }
 
        for (int ii=100;ii<=108;ii=ii+4) {
            result.add(new Integer(ii));
        }
        for (int jj=202;jj<=206;jj=jj+2) {
            result.add(new Integer(jj));
        }

        for (int kk=52;kk<=68;kk=kk+4) {
            result.add(new Integer(kk));
        }

        assertTrue("Wrong result EJBQL Customer.findByGoogCredit() (required:" + result + ", found:" +customers_id
                   + ")", isCollectionEqual(result, customers_id));
 
        // checkIsInitialState();  not necessary in this test
    }

    /**
     * 	Finding Customers having City = Minneapolis and STATE= MN
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.homeAddress.city = ?1 AND c.homeAddress.state = ?2 
     */
    public void testAnd2() throws Exception {
        Collection mplscustomers = customerhome.findByCity("Minneapolis","MN");

        ArrayList customers_id  = new ArrayList();
        Iterator customer = mplscustomers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();
        result.add(new Integer(206));
        result.add(new Integer(50));
        result.add(new Integer(56));
        result.add(new Integer(62));
        result.add(new Integer(68));
        result.add(new Integer(80));
        result.add(new Integer(86));
        result.add(new Integer(92));
        result.add(new Integer(98));
        result.add(new Integer(201));
        result.add(new Integer(202));
        result.add(new Integer(203));
        result.add(new Integer(204));
        result.add(new Integer(205));

        assertTrue("Wrong result EJBQL Customer.findByCity(Minneapolis,MN) (required:" + result + ", found:" +customers_id
                   + ")", isCollectionEqual(result, customers_id));
 
        //checkIsInitialState();not necessary in this test
    }
    /**
     * 	Finding Customer having a name exactly matching 'Joe Star' & 
     *  Finding Customers having a name like 'Jo S' (no wildcards) &
     *  Finding Customers having a name like 'Jo% S%' (with wildcards)
     *  Finding Customers having a name like 'Jo% S%' and living in MN
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.lastName = ?1 AND c.firstName = ?2 & 
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.lastName LIKE ?1 AND c.firstName LIKE ?2 &
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.lastName LIKE ?1 AND c.firstName LIKE ?2
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.lastName LIKE ?1 AND c.firstName LIKE ?2 AND c.homeAddress.state = ?3
     */
    public void testwildcards() throws Exception {
        // first test : exactly matching
        CustomerRemote customer = customerhome.findByExactName("Star","Joe");
        assertEquals("Wrong Customer for customerhome.findByExactName(Star,Joe);: ",new Integer(58), customer.getId());

        // second test : find with no wildcards
        Collection customers = customerhome.findByName("S","Jo");
        ArrayList customers_id  = new ArrayList();
        Iterator customerit1 = customers.iterator();
        while (customerit1.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customerit1.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();
        assertTrue("Wrong result EJBQL customerhome.findByName(S,Jo) (required:" + result + ", found:" +customers_id
                   + ")", isCollectionEqual(result, customers_id));

        // thirdth test : find with no wildcards
        customers = customerhome.findByName("S%","Jo%");
        customers_id  = new ArrayList();
        Iterator customerit2 = customers.iterator();
        while (customerit2.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customerit2.next();
            customers_id.add(customer_remote.getId());
        }
        result = new ArrayList();
        result.add(new Integer(50));
        result.add(new Integer(53));
        result.add(new Integer(55));
        result.add(new Integer(58));
        result.add(new Integer(65));
        result.add(new Integer(68));
        for (int i=80;i<=99;i++) {
            result.add(new Integer(i));
        }
        assertTrue("Wrong result EJBQL customerhome.findByName(S%,Jo%) (required:" + result + ", found:" +customers_id
                   + ")", isCollectionEqual(result, customers_id));

        customers = customerhome.findByNameAndState("S%","Jo%","MN");
        customers_id  = new ArrayList();
        Iterator customerit3 = customers.iterator();
        while (customerit3.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customerit3.next();
            customers_id.add(customer_remote.getId());
        }
        result = new ArrayList();
        result.add(new Integer(50));
        result.add(new Integer(68));
        result.add(new Integer(58));
        for (int i=80;i<=99;i=i+2) {
            result.add(new Integer(i));
        }
        assertTrue("Wrong result EJBQL customerhome.findByNameAndState(S%,Jo%,MN) (required:" + result + ", found:" +customers_id
                   + ")", isCollectionEqual(result, customers_id));


        // checkIsInitialState(); not necessary in this test
    }

    /**
     * 	Retrieve a collection of all cabins on deck 3
     *  SELECT OBJECT(c) FROM Cabin as c WHERE c.deckLevel = ?1
     */
    public void testEgalInteger() throws Exception {
        Collection cabins = cabinhome.findAllOnDeckLevel(new Integer(3));

        ArrayList cabins_id  = new ArrayList();
        Iterator cabin = cabins.iterator();
        while (cabin.hasNext()) {
            CabinRemote cabin_remote = (CabinRemote)cabin.next();
            cabins_id.add(cabin_remote.getId());
        }
        ArrayList result = new ArrayList();
        result.add(new Integer(21300));
        result.add(new Integer(21310));
        result.add(new Integer(21301));
        result.add(new Integer(21311));

        assertTrue("Wrong result EJBQL  cabinhome.findAllOnDeckLevel(new Integer(3))(required:" + result + ", found:" +cabins_id+
                   ")", isCollectionEqual(result,cabins_id));
 
        // checkIsInitialState();  not necessary in this test
    }

    /**
     * 	Findng Customers Living in Warm Climates
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.homeAddress.state IN ('FL','TX','AZ','CA')
     */
    public void testInWhere() throws Exception {
        Collection customers = customerhome.findInHotStates();
	
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();
        for (int i=51;i<=69;i=i+2) {
            result.add(new Integer(i));
        }
        for (int j=100;j<=109;j=j+2) {
            result.add(new Integer(j));
        }       
        for (int k=81;k<=99;k=k+2) {
            result.add(new Integer(k));
        }    

        assertTrue("Wrong result EJBQL customerhome.findInHotStates() (required:" + result + ", found:" +customers_id+
                   ")", isCollectionEqual(result,customers_id));
 
        // checkIsInitialState();  not necessary in this test
    }

    /**
     * 	Finding Customers With No Reservations
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.reservations IS EMPTY
     */
    public void testIsEmpty() throws Exception {
        Collection customers = customerhome.findWithNoReservations();
	
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();
        for (int i=51;i<=69;i=i+2) {
            result.add(new Integer(i));
        }
        for (int j=100;j<=109;j=j+2) {
            result.add(new Integer(j));
        }       
        for (int k=81;k<=99;k=k+2) {
            result.add(new Integer(k));
        }    

        assertTrue("Wrong result EJBQL customerhome.findWithNoReservations() (required:" + result + ", found:" +customers_id+
                   ")", isCollectionEqual(result,customers_id));
 
        //checkIsInitialState(); not necessary in this test
    }

    /**
     * 	Finding Customers On Alaska Cruise
     *  SELECT OBJECT(c) FROM jt2_Customer cust, Cruise cr
     *  WHERE cr = ?1 AND cust MEMBER OF cr.reservations.customers
     */
    public void testMemberOfNavigate() throws Exception {
        CruiseRemote crA = cruisehome.findByName("Alaska Cruise");
        Collection customers = customerhome.callFindOnCruise(crA.getId());
	
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();
        result.add(new Integer(201)); 
        result.add(new Integer(202)); 
        result.add(new Integer(203)); 
        result.add(new Integer(204)); 
        result.add(new Integer(205)); 
        result.add(new Integer(206)); 
        result.add(new Integer(52)); 
        result.add(new Integer(55)); 
        result.add(new Integer(58)); 
        result.add(new Integer(61)); 
        result.add(new Integer(64)); 
        result.add(new Integer(67)); 

        assertTrue("Wrong result EJBQL customerhome.findOnCruise(crA) (required:" + result + ", found:" +customers_id+
                   ")", isCollectionEqual(result,customers_id));
 
        // checkIsInitialState(); not necessary in this test
    }

    /**
     * 	Finding ship by tonnage
     *  SELECT OBJECT(c) FROM jt2_Ship s
     *  WHERE s.tonnage = ?1
     */
    public void testEgalDouble() throws Exception {
        Collection ships = shiphome.findByTonnage(new Double(100000));
	
        ArrayList ships_id  = new ArrayList();
        Iterator ship = ships.iterator();
        while (ship.hasNext()) {
            ShipRemote ship_remote = (ShipRemote)ship.next();
            ships_id.add(ship_remote.getId());
        }
        ArrayList result = new ArrayList();
        result.add(new Integer(7)); 
 

        assertTrue("Wrong result EJBQL shiphome.findByTonnage(new Double(100000)) (required:" + result + ", found:" +ships_id+ ")", isCollectionEqual(result,ships_id));
 
        // checkIsInitialState(); not necessary in this test
    }


    /**
     * 	Finding ship by tonnage
     *  SELECT OBJECT(c) FROM jt2_Ship s
     *  WHERE s.tonnage BETWEEN ?1 AND ?2
     */
    public void testBetweenDouble() throws Exception {
        Collection ships = shiphome.findByTonnage(new Double(100000),new Double(130000));
	
        ArrayList ships_id  = new ArrayList();
        Iterator ship = ships.iterator();
        while (ship.hasNext()) {
            ShipRemote ship_remote = (ShipRemote)ship.next();
            ships_id.add(ship_remote.getId());
        }
        ArrayList result = new ArrayList();
        result.add(new Integer(7)); 
        result.add(new Integer(8)); 
        result.add(new Integer(9));  
        result.add(new Integer(10)); 

        assertTrue("Wrong result EJBQL shiphome.findByTonnage(new Double(100000),new Double(130000)) (required:" + result + ", found:" +ships_id+ ")", isCollectionEqual(result,ships_id));
 
        // checkIsInitialState(); not necessary in this test
    }

    /**
     * 	Finding all city in which the credit card companies that distribute credit cards used by customers
     *  The address information of any credit card companies whose cards are not currently used by customers
     *  won't be includes in the result  
     *  
     *  SELECT c.creditCard.creditCompany.address.city FROM jt2_Customer As c
     */
    public void testNavigate() throws Exception {


        Collection customers = customerhome.findAllCreditCardAddress("Minneapolis");
	
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();
        result.add(new Integer(80)); 
        result.add(new Integer(84)); 
        result.add(new Integer(88));  
        result.add(new Integer(92));  
        result.add(new Integer(96)); 

        assertTrue("Wrong result EJBQL customerhome.findAllCreditCardAddress() (required:" + result + ", found:" +customers_id+
                   ")", isCollectionEqual(result,customers_id));
 
        checkIsInitialState();

    }

    /**
     * 	Finding all Customer with reservations (there will be duplication in result if customer has more than one reservation
     *  
     *  SELECT OBJECT(c) FROM jt2_Reservation res, IN(res.customers) c
     */
    public void testInFrom() throws Exception {

        Collection customers = customerhome.findAllCustomersWithReservation();
	
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();

        result.add(new Integer(201));
        result.add(new Integer(201));
        result.add(new Integer(202));
        result.add(new Integer(202));
        result.add(new Integer(203));
        result.add(new Integer(203));
        result.add(new Integer(204));
        result.add(new Integer(204));
        result.add(new Integer(205));
        result.add(new Integer(205));
        result.add(new Integer(206));
        result.add(new Integer(206));
        result.add(new Integer(50));
        result.add(new Integer(52));
        result.add(new Integer(53));
        result.add(new Integer(55));
        result.add(new Integer(56));
        result.add(new Integer(58));
        result.add(new Integer(59));
        result.add(new Integer(61));
        result.add(new Integer(62));
        result.add(new Integer(64));
        result.add(new Integer(65));
        result.add(new Integer(67));

        assertTrue("Wrong result EJBQL customerhome.findAllCustomersWithReservation (required:" + result + ", found:" +customers_id+
                   ")", isCollectionEqual(result,customers_id));
 
        // checkIsInitialState();  not necessary in this test
    }

    /**
     * 	Finding all Customer with reservations (there will not be duplication in result)
     *  
     *  SELECT DISTINCT OBJECT(c) FROM jt2_Reservation res, IN(res.customers) c
     */
    public void testDistinctInFrom() throws Exception {

        Collection customers = customerhome.findAllCustomersWithReservationDistinct();
	
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();

        result.add(new Integer(201));
        result.add(new Integer(202));
        result.add(new Integer(203));
        result.add(new Integer(204));
        result.add(new Integer(205));
        result.add(new Integer(206));
        result.add(new Integer(50));
        result.add(new Integer(52));
        result.add(new Integer(53));
        result.add(new Integer(55));
        result.add(new Integer(56));
        result.add(new Integer(58));
        result.add(new Integer(59));
        result.add(new Integer(61));
        result.add(new Integer(62));
        result.add(new Integer(64));
        result.add(new Integer(65));
        result.add(new Integer(67));

        assertTrue("Wrong result EJBQL customerhome.findAllCustomersWithReservationDistinct (required:" + result + ", found:" +customers_id+
                   ")", isCollectionEqual(result,customers_id));
 
        // checkIsInitialState(); not necessary in this test
    }

    /**
     * 	Finding Customer by address
     *  
     * 	SELECT OBJECT(c) FROM jt2_Customer c WHERE c.homeAddress = ?1
     */
    public void testBean() throws Exception {

        AddressRemote add = addresshome.createAddress( "1080 Elm Street", "Minneapolis", "MN", "55401");

        Collection customers = customerhome.findByAddress(add);
	
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();

        result.add(new Integer(80));

        assertTrue("Wrong result EJBQL customerhome.findByAddress(add)(required:" + result + ", found:" +customers_id+
                   ")", isCollectionEqual(result,customers_id));
 
        // checkIsInitialState(); not necessary in this test
    }

/**
     * 	Finding Customer by address
     *  
     * 	SELECT OBJECT(c) FROM jt2_Customer c WHERE c.homeAddress = ?1
     */
    public void testBeanNull() throws Exception {

        Collection customers = customerhome.findByAddress(null);
	
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();

        assertTrue("Wrong result EJBQL customerhome.findByAddress(add)(required:" + result + ", found:" +customers_id+
                   ")", isCollectionEqual(result,customers_id));
 
        // checkIsInitialState(); not necessary in this test
    }

    /**
     * 	Finding All Customer
     *  
     * 	SELECT OBJECT(c) FROM jt2_Customer AS c
     */
    public void testFindCustomerAll() throws Exception {

        Collection customers = customerhome.findCustomersAll();
	
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            customers_id.add(customer_remote.getId());
        }
        ArrayList result = new ArrayList();
        
        for (int i=80;i<=99;i++) {
            result.add(new Integer(i));
        }
        for (int j=201;j<=206;j++) {
            result.add(new Integer(j));
        }       
        for (int k=50;k<=69;k++) {
            result.add(new Integer(k));
        }    
        for (int l=100;l<=109;l++) {
            result.add(new Integer(l));
        }
        
        assertTrue("Wrong result EJBQL customerhome.findCustomersAll()(required:" + result + ", found:" +customers_id+
                   ")", isCollectionEqual(result,customers_id));
 
        // checkIsInitialState(); not necessary in this test
    }

    /**
     * 	Finding Customers by AddressLocal
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.homeAddress = ?1
     */
    public void testBeanLocal() throws Exception {
        Collection customers = customerhome.callFindByAddressLocal(new Integer(10));
	
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            customers_id.add((Integer)customer.next());
        }
        ArrayList result = new ArrayList();
        result.add(new Integer(80)); 

        assertTrue("Wrong result EJBQL customerhome.callFindByAddressLocal(10) (required:" + result + ", found:" +customers_id+
                   ")", isCollectionEqual(result,customers_id));
 
        // checkIsInitialState(); not necessary in this test
    }

    /**
     * 	Finding Customers by AddressLocal
     *  SELECT OBJECT(c) FROM jt2_Customer c WHERE c.homeAddress = ?1
     */
    public void testBeanLocalNull() throws Exception {
        Collection customers = customerhome.callFindByAddressLocal(null);
	
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            customers_id.add((Integer)customer.next());
        }
        ArrayList result = new ArrayList();

        assertTrue("Wrong result EJBQL customerhome.callFindByAddressLocal(null) (required:" + result + ", found:" +customers_id+
                   ")", isCollectionEqual(result,customers_id));
 
        // checkIsInitialState(); not necessary in this test
    }

    /**
     * Test a finder method
     * - which have no parameter and which have its name starts with 'findAll',
     * - but which not means "findAll".
     * SELECT OBJECT(c) FROM jt2_Customer c WHERE c.firstName LIKE 'Mike'
     */
    public void testFindAllMike() throws Exception {
        Collection customers = customerhome.findAllMike();
        ArrayList customers_id  = new ArrayList();
        Iterator customer = customers.iterator();
        while (customer.hasNext()) {
            CustomerRemote customer_remote = (CustomerRemote)customer.next();
            assertEquals("findAllMike() wrong result: ",
                         "Mike", customer_remote.getName().getFirstName());
        }
    }

    /**
     * Select the lastName of a customer
     * SELECT c.lastName FROM jt2_Customer AS c WHERE c.id = ?1
     */
    public void testCustEjbSelectLastName() throws Exception {
        assertEquals("Wrong ejbSelectLastName for c.id=80: ",
                     "Smith80", customerhome.selectLastName(new Integer(80)));
    }

    /**
     * Select the city of the address of a customer
     * SELECT c.homeAddress.city FROM jt2_Customer AS c WHERE c.id = ?1
     */
    public void testCustEjbSelectAddrCity() throws Exception {
        assertEquals("Wrong ejbSelectAddrCity for c.id=80: ",
                     "Minneapolis", customerhome.selectAddrCity(new Integer(80)));
    }

    /**
     * Select the deckLevel of a cabin
     * SELECT c.deckLevel FROM jt2_Cabin AS c WHERE c.id = ?1
     */
    public void testCabinEjbSelectDeckLevel() throws Exception {
        assertEquals("Wrong ejbSelectDeckLevel for c.id=21100: ",
                     1, cabinhome.selectDeckLevel(new Integer(21100)));
    }

    /**
     * Select the firstName of alll the customers.
     * SELECT c.firstName FROM jt2_Customer AS c
     */
    public void testCustEjbSelectFirstName() throws Exception {
        Set lfn = customerhome.selectFirstName();
        Iterator ilfn = lfn.iterator();
        StringBuffer slfn = new StringBuffer();
        while (ilfn.hasNext()) {
            slfn = slfn.append((String) ilfn.next() + ",");
        }
        assertEquals("Wrong ejbSelectFirstName() number results (" + slfn.toString() + "): ",
                     6, lfn.size());
        if (!lfn.contains("John")
            || !lfn.contains("Mike")
            || !lfn.contains("Paul")
            || !lfn.contains("Ringo")
            || !lfn.contains("Roger")
            || !lfn.contains("Joe")) {
            fail("Wrong ejbSelectFirstName() result (" + slfn.toString()
                 + "): does not contain 'John', 'Mike' or 'Paul' or 'Ringo' or 'Roger' or 'Joe'");
        }
    }

    /**
     * Select the address of a customer
     * An ejbSelect with a local bean as result
     * SELECT c.homeAddress FROM jt2_Customer AS c WHERE c.id = ?1
     */
    public void testCustEjbSelectAddr() throws Exception {
        assertEquals("Wrong ejbSelectAddr for c.id=80: ",
                     new Integer(10), customerhome.selectAddr(new Integer(80)));
    }

    /**
     * Select the addresses of the company of the credit card of all the customers.
     * An ejbSelect with a collection of local bean as result
     * SELECT c.creditCard.creditCompany.address FROM jt2_Customer c
     */
    public void testCustEjbSelectAllCreditCartAddr() throws Exception {
        Collection addr = customerhome.selectAllCreditCardAddr();
        StringBuffer sAddr  = new StringBuffer();
        Iterator iAddr = addr.iterator();
        while (iAddr.hasNext()) {
            sAddr = sAddr.append(((Integer) iAddr.next()).toString() + ",");
        }
        ArrayList eAddr = new ArrayList();
        eAddr.add(new Integer(11));
        eAddr.add(new Integer(16));
        eAddr.add(new Integer(21));
        eAddr.add(new Integer(26));
        eAddr.add(new Integer(31));
        assertTrue("Wrong ejbSelectCreditCardAddr: (" + sAddr + "): ",
                   isCollectionEqual(eAddr, addr));
    }

    /**
     * Select the adresses of all 'roger' customers.
     * An ejbSelect with a collection of local bean as result
     * SELECT c.homeAddress FROM jt2_Customer c WHERE c.firstName = 'Roger'
     */
    public void testCustEjbSelectRogerAddr() throws Exception {
        Collection addr = customerhome.selectRogerAddr();
        StringBuffer sAddr  = new StringBuffer();
        Iterator iAddr = addr.iterator();
        while (iAddr.hasNext()) {
            sAddr = sAddr.append(((Integer) iAddr.next()).toString() + ",");
        }
        ArrayList eAddr = new ArrayList();
        eAddr.add(new Integer(45));
        eAddr.add(new Integer(50));
        eAddr.add(new Integer(55));
        eAddr.add(new Integer(60));
        assertTrue("Wrong ejbCustSelectRogerAddr: (" + sAddr + "): ",
                   isCollectionEqual(eAddr, addr));
    }


    public static Test suite() {
        return new TestSuite(F_EjbqlEC2.class);
    }

    public static void main (String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String s_arg = args[argn];
            Integer i_arg;
            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_EjbqlEC2(testtorun));
        }
    }
}

