/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.rmi.RemoteException;
import java.util.Enumeration;
import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.beans.ebasic.*;
import org.objectweb.jonas.jtests.util.JTestCase;
import javax.ejb.ObjectNotFoundException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**
 * This is an advanced test suite for home interface on pk auto incremented CMP1
 */
public class F_PkAutoEC extends A_Cmp2Util {

    private static String BEAN_HOME_PKAUTO = "pkautocmp1Home";

    protected static pkautoHome pkautocmp1home = null;

    public F_PkAutoEC(String name) {
        super(name);
    }

    protected void setUp() {
    	super.setUp();
        boolean ok = false;
        int nbtry = 0;
        while (!ok && nbtry < 3) {
            if (!isInit) {
                // load bean if not loaded yet
                useBeans("ebasic", true);
                // lookup home used in the tests
                try {
                    pkautocmp1home = (pkautoHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_PKAUTO),
                                                                              pkautoHome.class);
                } catch (NamingException e) {
                    fail("Cannot get bean home: " + e.getMessage());
                }
                // check if tables have been initialized
                try {
                    Collection bpk = pkautocmp1home.findAll();
                    if (bpk.isEmpty()) throw new Exception("Table vide");
                } catch (Exception e) {
                    // Make the initialization needed for the tests

                    try {
                        utx.begin(); 
                        for (int i=1;i<=10;i++) {
                            pkautocmp1home.create(10,20);
                        }

                    } catch (Exception i) {
                        i.printStackTrace(System.out);
                        fail("InitialState creation problem: "+i);
                    } finally {
                        try {
                            utx.commit();
                        } catch (Exception ii) {
                        }
                    }
                }
                isInit = true;
            }
            // Check that all is OK. Sometimes, a test has failed and has corrupted
            // the bean state in the database. We must unload and reload the bean then.
            nbtry++;
            try {
                if (initStateOK()) {
                    ok = true;
                }
            } catch (Exception e) {
                e.printStackTrace(System.out);
            }
            if (!ok) {
                isInit = false;
                unloadBeans("ebasic");
            }
        }
    }


    /*
     * Check that we are in the same state as after the tables creation for thoses beans A and B
     * (ie if it is the initial state)
     */
    boolean initStateOK() throws Exception {

        Collection bpk = pkautocmp1home.findAll();
        if (bpk.size()==10)
            return true;
        else
            return false;
    }

    /**
     * 	
     */
    public void testEmpty() throws Exception {
        checkIsInitialState();
    }
	
    /**
     */
    public static Test suite() {
        return new TestSuite(F_PkAutoEC.class);
    }

    public static void main (String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String s_arg = args[argn];
            Integer i_arg;
            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_PkAutoEC(testtorun));
        }
    }
}

